!! ----------------------------------------------------------------------------
!! *PROGRAM* ReadInpNCDF_ceilo_data
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/ReadInpNCDF_ceilo_data.f90
!!
!!
!! *LAST CHANGES*
!!
!!
!!  -May 14, 2014: A.T.   ReadInpNCDF_ceilo_data modified to include following quality control parameters
!!                        when processing input set from ceilometer data:qc_first_cbh, detection_status,
!!                        qc_detection_status, and valid_min,valid_max for first_cbh
!!
!!
!! *DESCRIPTION* 
!!
!! This program contains subroutines needed for reading the ceilometer netcdf data files
!! from ARM arhive.
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
subroutine ReadInpNCDF_dim_ceilo_data(InpFile,ceilo_data,status)
!
Use netcdf
Use typeSizes
Use write_messages
Use radar_mods
!
Implicit None
!
Character(len=*),Intent(in)                              :: InpFile
Type(sacr_ceilo_data),Intent(InOut)                      :: ceilo_data
Integer,Intent(out)                                      :: status
!
Integer                                                  :: ncid, iDim,length
Integer                                                  :: nDims,nVars
Character(len=nf90_max_name)                             :: name, err_msg
Integer, Allocatable, Dimension(:)                       :: dim_lengths
Character(len=nf90_max_name), Allocatable , Dimension(:) :: dim_names
Integer                                                  :: aa
!
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
Allocate(dim_names(1:nDims))
Allocate(dim_lengths(1:nDims))
!
! Get info on the dimensions
! Loop over the number of dimensions
!   
Do iDim = 1,nDims
!
status=nf90_inquire_dimension(ncid, idim, name=name, len=length)
!
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_dimension'
Goto 999
End If
!
dim_names(iDim)   = name
dim_lengths(iDim) = length
!
Select Case (name)
Case ('time')        ;  ceilo_data%nt   = length
Case ('range')       ;  aa              = length
Case ('string_len')  ;  aa              = length
   Case default
   status = 1
   err_msg = 'Unrecognised dimension: '
   Goto 999
End Select
enddo   ! iDim
!
Deallocate(dim_names)
Deallocate(dim_lengths)
!
999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=  nf90_close(ncid)
Endif
!
return
end subroutine ReadInpNCDF_dim_ceilo_data
!!
subroutine ReadInpNCDF_var_ceilo_data(InpFile,ceilo_data,status)
!
Use netcdf
Use typeSizes
Use write_messages
Use radar_mods
!
Implicit None
!
Character(len=*),Intent(in)                              :: InpFile
Type(sacr_ceilo_data),Intent(InOut)                      :: ceilo_data
Integer,Intent(out)                                      :: status
!
integer                                                  :: nDims, nVars
integer                                                  :: ncid, iVar
Character(len=nf90_max_name), Allocatable , Dimension(:) :: var_names
Character(len=nf90_max_name)                             :: name, err_msg
!
integer                                                  :: base_time
real*8,Dimension(:),Allocatable                          :: time_offset
real*4,Dimension(:),Allocatable                          :: first_cbh
integer*4,Dimension(:),Allocatable                       :: qc_first_cbh
integer*4,Dimension(:),Allocatable                       :: detection_status
integer*4,Dimension(:),Allocatable                       :: qc_detection_status
real*4                                                   :: alt ! alt above mean sea level
real*8,parameter                                         :: m999=-999.d0
integer                                                  :: nt
integer                                                  :: it
real*4                                                   :: valid_min,valid_max
!
!
nt=ceilo_data%nt
Allocate(first_cbh(nt),time_offset(nt),qc_first_cbh(nt),detection_status(nt),qc_detection_status(nt))
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
!
! Create arrays that contain the variables
Allocate(var_names(1:nVars))
!
Do  iVar=1,nVars
!
status= nf90_inquire_variable(ncid, iVar, name=name)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_variable: '//name
Goto 999
Endif
!
Select Case (Trim(Adjustl(name)))
!
Case ('first_cbh')
valid_min=999.e0 ; valid_max=999.e0
status=nf90_get_var(ncid,iVar,first_cbh)
status = nf90_get_att(ncid, iVar, "valid_min", valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: first_cbh'
Goto 999
End If
!
Case ('qc_first_cbh')
status=nf90_get_var(ncid,iVar,qc_first_cbh)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_first_cbh'
Goto 999
End If
!
Case ('detection_status')
status=nf90_get_var(ncid,iVar,detection_status)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: detection_status'
Goto 999
End If
!
Case ('qc_detection_status')
status=nf90_get_var(ncid,iVar,qc_detection_status)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_detection_status'
Goto 999
End If
!
Case ('time_offset')
status=nf90_get_var(ncid,iVar,time_offset)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: time_offset'
Goto 999
End If
!
Case ('base_time')
status=nf90_get_var(ncid,iVar,base_time)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: base_time'
Goto 999
End If
!
Case ('alt')
status=nf90_get_var(ncid,iVar,alt)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: alt'
Goto 999
End If

End Select
!
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var:'
Goto 999
End If
!
End Do
!
Deallocate(var_names)

999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=nf90_close(ncid)
Endif
!
!----------------------------------------------------------------------------------
!! input data managment
!! 
ceilo_data%alt=dble(alt)  ! alt above mean sea level
where (first_cbh < valid_min) first_cbh=-999.e0
where (first_cbh > valid_max) first_cbh=-999.e0
where (qc_first_cbh/=0) first_cbh=-999.e0
!
do it=1,nt
ceilo_data%time_s(it)=time_offset(it)+dble(base_time) ! time in second since 1970-1-1 0:00:00 0:00

  if (first_cbh(it) >=valid_min) Then
      if ((detection_status(it)>0).and.(detection_status(it)<4).and.(qc_detection_status(it)==0)) Then
         ceilo_data%first_cbh(it)=dble(first_cbh(it))! above ground level
      else
         ceilo_data%first_cbh(it)=m999 
      endif
   endif
!
enddo
!
! now all not valid values (with respect to the lowest cloud base determination) are set to -999.d0
!where(ceilo_data%first_cbh/=m999) 
!ceilo_data%qc_first_cbh=qc_first_cbh
!ceilo_data%detection_status=detection_status
!ceilo_data%qc_detection_status=qc_detection_status
!end where
!
!
Deallocate(first_cbh,qc_first_cbh,time_offset,detection_status,qc_detection_status)
!
return
end subroutine ReadInpNCDF_var_ceilo_data



