!! ----------------------------------------------------------------------------
!! *PROGRAM* ReadInpNCDF_mergedsonde_data
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/ReadInpNCDF_mergedsonde_data.f90
!!
!!
!! *LAST CHANGES*
!!
!!
!! *DESCRIPTION* 
!!
!! This program contains subroutines needed for reading the merged sounding netcdf data files
!! from ARM arhive.
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
!!
subroutine ReadInpNCDF_dim_mergedsonde_data(InpFile,mergedsonde,status)
!
Use netcdf
Use typeSizes
Use write_messages
Use radar_mods
!
Implicit None
!
Character(len=*),Intent(in)                              :: InpFile
Type(sacr_sounding_data),Intent(InOut)                   :: mergedsonde
Integer,Intent(out)                                      :: status
!
Integer                                                  :: ncid, iDim,length
Integer                                                  :: nDims,nVars
Character(len=nf90_max_name)                             :: name, err_msg
Integer, Allocatable, Dimension(:)                       :: dim_lengths
Character(len=nf90_max_name), Allocatable , Dimension(:) :: dim_names
Integer                                                  :: aa
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
Allocate(dim_names(1:nDims))
Allocate(dim_lengths(1:nDims))
!
! Get info on the dimensions
! Loop over the number of dimensions
!   
Do iDim = 1,nDims
!
status=nf90_inquire_dimension(ncid, idim, name=name, len=length)
!
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_dimension'
Goto 999
End If
!
dim_names(iDim)   = name
dim_lengths(iDim) = length
!
Select Case (name)
Case ('time')    ;  mergedsonde%nt   = length
Case ('height')  ;  mergedsonde%sounde   = length
   Case default
   status = 1
   err_msg = 'Unrecognised dimension: '
   Goto 999
End Select
enddo   ! iDim
!
999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=  nf90_close(ncid)
Endif
!
return
end subroutine ReadInpNCDF_dim_mergedsonde_data
!
subroutine ReadInpNCDF_var_mergedsonde_data(InpFile,time_str,mergedsonde,status)
!
Use netcdf
Use typeSizes
Use write_messages
Use radar_mods
!
Implicit None
!
Character(len=*),Intent(in)                              :: InpFile
Character(len=*),Intent(in)                              :: time_str ! sacr time at the middle of the scan YYYYMMDDHHSS
Type(sacr_sounding_data),Intent(InOut)                   :: mergedsonde
Integer,Intent(out)                                      :: status
!
integer                                                  :: nDims, nVars
integer                                                  :: ncid, iVar
Character(len=nf90_max_name), Allocatable , Dimension(:) :: var_names
Character(len=nf90_max_name)                             :: name, err_msg
!
real*8,Dimension(:),Allocatable    :: time_s
real*4,Dimension(:),Allocatable    :: alt
real*4,Dimension(:,:),Allocatable  :: temp,rh,bar_pres,wspd, wdir
real*4,Dimension(:,:),Allocatable  :: qc_temp,qc_rh,qc_bar_pres,qc_wspd, qc_wdir

real*4                             :: temp_valid_min,temp_valid_max
real*4                             :: rh_valid_min,rh_valid_max
real*4                             :: bar_pres_valid_min,bar_pres_valid_max
real*4                             :: wspd_valid_min,wspd_valid_max
real*4                             :: wdir_valid_min,wdir_valid_max
!
integer                            :: nz,nt
integer                            :: iz,it
Character(len=3)                   :: rlocation
real*8,parameter                   :: m999=-999.d0
!
integer                            ::  ipr ! mathed time index that correspond to the nearest radar scan time 
!
nt=mergedsonde%nt
nz=mergedsonde%sounde
!
Allocate(alt(nz),time_s(nt))
Allocate(temp(nz,nt),rh(nz,nt),bar_pres(nz,nt),wspd(nz,nt), wdir(nz,nt))
Allocate(qc_temp(nz,nt),qc_rh(nz,nt),qc_bar_pres(nz,nt),qc_wspd(nz,nt), qc_wdir(nz,nt))
!
alt=real(m999) ; time_s=m999
temp=real(m999);rh=real(m999);bar_pres=real(m999);wspd=real(m999);wdir=real(m999)
qc_temp=int(m999);qc_rh=int(m999);qc_bar_pres=int(m999);qc_wspd=int(m999);qc_wdir=int(m999)
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
! Create arrays that contain the variables
Allocate(var_names(1:nVars))
!
Do  iVar=1,nVars
!
status= nf90_inquire_variable(ncid, iVar, name=name)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_variable: '//name
Goto 999
Endif
!
!
Select Case (Trim(Adjustl(name)))
!
!
Case ('time')
status=nf90_get_var(ncid,iVar,time_s)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: time'
Goto 999
End If
!
Case ('wspd')
status=nf90_get_var(ncid,iVar,wspd)
status = nf90_get_att(ncid, iVar, "valid_min", wspd_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", wspd_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: wspd'
Goto 999
End If
!
Case ('qc_wspd')
status=nf90_get_var(ncid,iVar,qc_wspd)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_wspd'
Goto 999
EndIf
!
Case ('wdir')
status=nf90_get_var(ncid,iVar,wdir)
status = nf90_get_att(ncid, iVar, "valid_min", wdir_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", wdir_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: wdir'
Goto 999
End If
!
Case ('qc_wdir')
status=nf90_get_var(ncid,iVar,qc_wdir)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_wdir'
Goto 999
EndIf
!
Case ('temp')
status=nf90_get_var(ncid,iVar,temp)
status = nf90_get_att(ncid, iVar, "valid_min", temp_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", temp_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: temp'
Goto 999
End If
!
Case ('qc_temp')
status=nf90_get_var(ncid,iVar,qc_temp)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_temp'
Goto 999
End If
!
Case ('bar_pres')
status=nf90_get_var(ncid,iVar,bar_pres)
status = nf90_get_att(ncid, iVar, "valid_min", bar_pres_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", bar_pres_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: bar_pres'
Goto 999
End If
!
Case ('qc_bar_pres')
status=nf90_get_var(ncid,iVar,qc_bar_pres)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_bar_pres'
Goto 999
End If
!
Case ('rh')
status=nf90_get_var(ncid,iVar,rh)
status = nf90_get_att(ncid, iVar, "valid_min", rh_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", rh_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: rh'
Goto 999
End If
!
Case ('qc_rh')
status=nf90_get_var(ncid,iVar,qc_rh)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_rh'
Goto 999
End If
!
Case ('height')
status=nf90_get_var(ncid,iVar,alt)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: height'
Goto 999
End If
!
End Select
!
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var:'
Goto 999
End If
!
End Do
!
999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=nf90_close(ncid)
Endif
!
!----------------------------------------------------------------------------------
!! input data managment
!! 
where(wspd < wspd_valid_min) wspd=-999.e0
where(wspd > wspd_valid_max) wspd=-999.e0
where(qc_wspd/=0) wspd=-999.e0
!!
where(wdir < wdir_valid_min) wdir=-999.e0
where(wdir > wdir_valid_max) wdir=-999.e0
where(qc_wdir/=0) wdir=-999.e0
!!
where(temp < temp_valid_min) temp=-999.e0
where(temp > temp_valid_max) temp=-999.e0
where(qc_temp/=0) temp=-999.e0
!!
where(bar_pres < bar_pres_valid_min) bar_pres=-999.e0
where(bar_pres > bar_pres_valid_max) bar_pres=-999.e0
where(qc_bar_pres/=0) bar_pres=-999.e0
!!
where(rh < rh_valid_min) rh=-999.e0
where(rh > rh_valid_max) rh=-999.e0
where(qc_rh/=0) rh=-999.e0
!!
!!
!!

!! Identify the closest sounding to the scan
!! get ipr - the index of the closest sounding to the scan and the time interval
!! between radar scan and the soundings 
!
call GetIndexOfTheClosestSounding(nt,time_s,time_str,ipr,mergedsonde%dt_sounding_scan)
!
! define output variable according to output sacr_sounding_data structure
mergedsonde%wf(:)=dble(wspd(:,ipr))
mergedsonde%wd(:)=dble(wdir(:,ipr))
mergedsonde%Temperature(:)=dble(temp(:,ipr))
mergedsonde%pressure(:)=dble(bar_pres(:,ipr))
mergedsonde%relative_hum(:)=dble(rh(:,ipr))



!! match units to those in sounding files
!! pressure  kPa->hPa
mergedsonde%pressure=mergedsonde%pressure*10.d0 ! pressure  kPa->hPa
mergedsonde%altitude=dble(alt)*1.d+3           ! height km->m
!!
!!
!! change wind direction;
!! in input arm files North is zero degrees. Convention here: East is zero degrees.
!
where(mergedsonde%wd==m999) wdir(:,ipr)=real(m999)
!

!! wind direction change 
it=ipr ! matched time index
do iz=1,nz
if ((wdir(iz,it)>=0).and.(wdir(iz,it)<=90)) Then
mergedsonde%wd(iz)=90.d0-dble(wdir(iz,it))
else if ((wdir(iz,it)>90).and.(wdir(iz,it)<=360)) Then
mergedsonde%wd(iz)=360.d0-(dble(wdir(iz,it))-90.d0)
else
mergedsonde%wd(iz)=m999 
endif
enddo
!
Deallocate(wspd,wdir,temp,bar_pres,rh,alt,time_s)
Deallocate(qc_wspd,qc_wdir,qc_temp,qc_bar_pres,qc_rh)
!
return
end subroutine ReadInpNCDF_var_mergedsonde_data
!!
subroutine GetIndexOfTheClosestSounding(nt,time_s,time_string,ipr,dt_sounding_scan)
Implicit None
integer, Intent(in)                :: nt ! number of vertical profiles in mergedsonde data file
real*8, Intent(in)                 :: time_s(nt)  ! mergedsonde time in seconds since midnight
Character(len=*),Intent(In)        :: time_string ! sacr time at the middle of the scan YYYYMMDDHHSS
integer, Intent(Out)               :: ipr  ! index in time_s(1:nt) closest to the time YYYYMMDDHHSS
real*8, Intent(Out)                :: dt_sounding_scan ! [sec] time interval between sounding time and sacr scan time
!
integer                            :: true_itime
character(len=2)                   :: hh,mm,ss
integer                            :: ihh,imm,iss
integer                            :: it
integer                            :: diff,work
!
!! get true_itime :sacr time scan in seconds from midnight 
hh=time_string(10:11)
mm=time_string(12:13)
ss=time_string(14:15)
read (hh,*) ihh
read (mm,*) imm
read (ss,*) iss
true_itime=ihh*3600 + imm*60 + iss 
!
! find the index of the closest sounding time to the sacr scanning time
diff=abs(true_itime-time_s(1))
ipr=1
do it=1,nt
   work=abs(true_itime-time_s(it))
   if (work<diff) then
      ipr=it
      diff=work
   endif
enddo
!
dt_sounding_scan=dble(abs(true_itime-time_s(ipr)))
!
return
end subroutine GetIndexOfTheClosestSounding



