!! ----------------------------------------------------------------------------
!! *PROGRAM* ReadInpNCDF_raw_data
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/ReadInpNCDF_raw_data.f90
!!
!!
!! *LAST CHANGES*
!!
!!
!!  -May 28, 2014: A.T. sweep_start_ray_index found not correct for vptmo, TMP site.
!!                      This index is redifined for this special case. 
!!
!!  -May 26, 2014: A.T. Included possibility of coorecting sweep indices for "ppimo" mode.
!! 
!!  -May 22, 2014: A.T. Excluded calls for get_numb_scans and correct_sweep_indices
!!                      and includded call for new subroutine CorrectSweepIndices.
!!                      The Allocate/Deallocate structure modified to account for
!!                      the  dimension mismatch in the scan number for allaffected
!!                      variables.
!!
!!  -May 20, 2014: A.T. Variable fixed_angle included as part of input but found not realible.
!!                      Values may not be correct.
!!
!!
!! *DESCRIPTION* 
!!
!! This program contains subroutines needed for reading the scanning radar netcdf data files
!! from ARM archive. It also contains the subroutine for reading the climatological noise data
!! files. Note that variables sweep_start_ray_index and sweep_end_ray_index are not properly 
!! defined in the scanning radar netcdf data files and they are to be corrected in near future 
!! in the new reprocessed ARM scanning radar netcdf data files. In mean time these indices are 
!! re-calculated here. 
!! 
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
subroutine ReadInpNCDF_dim_raw_data(InpFile,raw_data,status)
!
Use netcdf
Use typeSizes
Use write_messages
Use radar_mods
!
Implicit None
!
Character(len=*),Intent(in)                              :: InpFile
Type(sacr_raw_data),Intent(inout)                        :: raw_data
Integer,Intent(out)                                      :: status
Integer                                                  :: ncid, iDim,length
Integer                                                  :: nDims,nVars
Character(len=nf90_max_name)                             :: name, err_msg
Integer, Allocatable, Dimension(:)                       :: dim_lengths
Character(len=nf90_max_name), Allocatable , Dimension(:) :: dim_names
Integer                                                  :: aa
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
Allocate(dim_names(1:nDims))
Allocate(dim_lengths(1:nDims))
!
! Get info on the dimensions
! Loop over the number of dimensions
!   
Do iDim = 1,nDims
!
status=nf90_inquire_dimension(ncid, idim, name=name, len=length)
!
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_dimension'
Goto 999
End If
!
dim_names(iDim)   = name
dim_lengths(iDim) = length
!
Select Case (name)
Case ('time')      ;  raw_data%ntimes = length
Case ('range')     ;  raw_data%ngates = length
Case ('sweep')     ;  raw_data%nsweep = length
Case ('frequency') ;  raw_data%nfrequency  = length
Case ('r_calib')   ;         aa   = length
Case ('string_length_4')  ;  aa   = length
Case ('string_length_6')  ;  aa   = length
Case ('string_length_10') ;  aa   = length
Case ('string_length_21') ;  aa   = length
Case ('string_length_24') ;  aa   = length
Case ('string_length_34') ;  aa   = length
   Case default
   status = 1
   err_msg = 'Unrecognised dimension: '
   Goto 999
End Select
enddo   ! iDim
!
raw_data%one=1
!
999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=  nf90_close(ncid)
Endif
!
return
end subroutine ReadInpNCDF_dim_raw_data
!
subroutine ReadInpNCDF_var_raw_data(InpFile,raw_data,rad_syst,sacr_param,status)
!
Use netcdf
Use typeSizes
Use radar_mods
Use write_messages
!                    
Implicit None
!
Character(len=*),Intent(in)                    :: InpFile
Type(sacr_raw_data),Intent(inout)              :: raw_data
Type(sacr_system_constants),Intent(InOut)      :: rad_syst
Type(sacr_parameters),Intent(In)               :: sacr_param
Integer, Intent(out)                           :: status
!
integer                                                  :: nDims, nVars 
integer                                                  :: ncid, iVar
Character(len=nf90_max_name), Allocatable , Dimension(:) :: var_names
Character(len=nf90_max_name)                             :: name, err_msg
!
integer                                                  :: one,nt,ng,nsweep,nfrequency
integer                                                  :: inp_base_time
Real*8,Dimension(:),Allocatable                          :: inp_offset_time
Real*4,Dimension(:),Allocatable                          :: inp_range
Real*4,Dimension(:),Allocatable                          :: inp_azimuth
Real*4,Dimension(:),Allocatable                          :: inp_elevation 
Integer*2,Dimension(:,:),Allocatable                     :: inp_DBZ
Integer*2,Dimension(:,:),Allocatable                     :: inp_VELO
Integer*2,Dimension(:,:),Allocatable                     :: inp_SNR
Integer*2,Dimension(:,:),Allocatable                     :: inp_LDR
Integer*2,Dimension(:,:),Allocatable                     :: inp_WIDTH
Integer*4,Dimension(:),Allocatable                       :: inp_numb_scans
Integer*4,Dimension(:),Allocatable                       :: inp_sweep_start_ray_index
Integer*4,Dimension(:),Allocatable                       :: inp_sweep_end_ray_index
Real*4,Dimension(:),Allocatable                          :: inp_frequency
Real*4,Dimension(:),Allocatable                          :: inp_nyquist_velo
!
Real*4,Dimension(:),Allocatable                          :: fixed_angle
Integer                                                  :: w1,w2                                            
!
Real*4                                                   :: inp_latitude
Real*4                                                   :: inp_longitude
Real*4                                                   :: inp_altitude_sea_level
Real*8                                                   :: inp_altitude_from_ground
!
Real*4                                                   :: scale_factor,add_offset
integer                                                  :: it
Real*4                                                   :: d1,d2,d3
Real*4,Dimension(:),Allocatable                          :: inp_azimuth1,awork
Integer,Dimension(:),Allocatable                         :: ij1,ij2
Integer                                                  :: modID ! =0 (fixedazimuth,variable elevation; =1 fixed elevation, variable azimuth )
!
!
one=raw_data%one ; nt=raw_data%ntimes ; ng=raw_data%ngates
nsweep=raw_data%nsweep ; nfrequency=raw_data%nfrequency

Allocate(inp_offset_time(nt),inp_range(ng),inp_azimuth(nt),inp_elevation(nt), &
         inp_DBZ(ng,nt),inp_VELO(ng,nt),inp_SNR(ng,nt), inp_LDR(ng,nt), inp_WIDTH(ng,nt), &
         inp_numb_scans(nsweep),inp_sweep_start_ray_index(nsweep),inp_sweep_end_ray_index(nsweep),&
         inp_frequency(nfrequency),inp_nyquist_velo(nt))
!        inp_longitude,inp_altitude_sea_level          

Allocate(fixed_angle(nsweep))
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
! Create arrays that contain the variables
Allocate(var_names(1:nVars))
!
Do  iVar=1,nVars
!
status= nf90_inquire_variable(ncid, iVar, name=name)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_variable: '//name
Goto 999
Endif
!
!
Select Case (Trim(Adjustl(name)))
!
Case ('fixed_angle')
status=nf90_get_var(ncid,iVar,fixed_angle)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: fixed_angle'
Goto 999
End If
!
!Case ('antenna_transition')
!status=nf90_get_var(ncid,iVar,antenna_transition)
!If (status/=0) Then
!err_msg = 'Error in my_nf90_get_var: antenna_transition'
!Goto 999
!End If
!
Case ('base_time')
status=nf90_get_var(ncid,iVar,inp_base_time)
raw_data%base_time(1)=dble(inp_base_time)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: base_time'
Goto 999
End If
!
Case ('time_offset')
status=nf90_get_var(ncid,iVar,inp_offset_time)
raw_data%offset_time=inp_offset_time
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: time_offset'
Goto 999
End If
!
Case ('range')
status=nf90_get_var(ncid,iVar,inp_range)
raw_data%range=dble(inp_range)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: range'
Goto 999
End If
!
Case('azimuth')
status=nf90_get_var(ncid,iVar,inp_azimuth)
raw_data%azimuth=dble(inp_azimuth)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: azimuth'
Goto 999
End If
!
Case ('elevation')
status=nf90_get_var(ncid,iVar,inp_elevation)
raw_data%elevation=dble(inp_elevation)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: elevation'
Goto 999
End If
!
Case ('reflectivity')
scale_factor=999.e0; add_offset=999.e0
status = nf90_get_var(ncid,iVar,inp_DBZ)
status = nf90_get_att(ncid, iVar, "scale_factor", scale_factor)
status = nf90_get_att(ncid, iVar, "add_offset",   add_offset)
rad_syst%dbz_fact=dble(scale_factor) ; rad_syst%dbz_offset=dble(add_offset)
raw_data%DBZ=dble(scale_factor)*dble(inp_DBZ)+dble(add_offset)
WHERE (raw_data%DBZ<-7000) raw_data%DBZ=-999.d0
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: reflectivity'
Goto 999
End If
!
Case ('mean_doppler_velocity')
scale_factor=999.e0; add_offset=999.e0
status = nf90_get_var(ncid,iVar,inp_VELO)
status = nf90_get_att(ncid, iVar, "scale_factor", scale_factor)
status = nf90_get_att(ncid, iVar, "add_offset",   add_offset)
rad_syst%velo_fact=dble(scale_factor) ; rad_syst%velo_offset=dble(add_offset)
raw_data%VELO=(dble(scale_factor)*dble(inp_VELO)+dble(add_offset))
WHERE (raw_data%VELO<-7000) raw_data%VELO=-999.d0
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: mean_doppler_velocity'
Goto 999
End If
!
Case ('snr')
scale_factor=999.e0; add_offset=999.e0
status = nf90_get_var(ncid,iVar,inp_SNR)
status = nf90_get_att(ncid, iVar, "scale_factor", scale_factor)
status = nf90_get_att(ncid, iVar, "add_offset",   add_offset)
rad_syst%snr_fact=dble(scale_factor) ; rad_syst%snr_offset=dble(add_offset)
raw_data%SNR=dble(scale_factor)*dble(inp_SNR)+dble(add_offset)
WHERE (raw_data%SNR<-7000) raw_data%SNR=-999.d0
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: snr'
Goto 999
End If
!
Case ('linear_depolarization_ratio')
scale_factor=999.e0; add_offset=999.e0
status = nf90_get_var(ncid,iVar,inp_LDR)
status = nf90_get_att(ncid, iVar, "scale_factor", scale_factor)
status = nf90_get_att(ncid, iVar, "add_offset",   add_offset)
rad_syst%ldr_fact=dble(scale_factor) ; rad_syst%ldr_offset=dble(add_offset)
raw_data%LDR=dble(scale_factor)*dble(inp_LDR)+dble(add_offset)
WHERE (raw_data%LDR<-7000) raw_data%LDR=-999.d0
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: linear_depolarization_ratio'
Goto 999
End If
!
Case ('spectral_width')
scale_factor=999.e0; add_offset=999.e0
status = nf90_get_var(ncid,iVar,inp_WIDTH)
status = nf90_get_att(ncid, iVar, "scale_factor", scale_factor)
status = nf90_get_att(ncid, iVar, "add_offset",   add_offset)
rad_syst%sw_fact=dble(scale_factor) ; rad_syst%sw_offset=dble(add_offset)
raw_data%WIDTH=dble(scale_factor)*dble(inp_WIDTH)+dble(add_offset)
WHERE (raw_data%WIDTH<-7000) raw_data%WIDTH=-999.d0
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: spectral_width'
Goto 999
End If
!
Case('sweep_number')
status=nf90_get_var(ncid,iVar,inp_numb_scans)
raw_data%numb_scans=int(inp_numb_scans)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: sweep_number'
Goto 999
End If
!
Case('sweep_start_ray_index')
status=nf90_get_var(ncid,iVar,inp_sweep_start_ray_index)
raw_data%sweep_start_ray_index=int(inp_sweep_start_ray_index)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: sweep_start_ray_index'
Goto 999
End If
!
Case('sweep_end_ray_index')
status=nf90_get_var(ncid,iVar,inp_sweep_end_ray_index)
raw_data%sweep_end_ray_index=int(inp_sweep_end_ray_index)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: sweep_end_ray_index'
Goto 999
End If
!
Case('frequency')
status=nf90_get_var(ncid,iVar,inp_frequency)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: frequency'
Goto 999
End If
!
Case('nyquist_velocity')
status=nf90_get_var(ncid,iVar,inp_nyquist_velo)
rad_syst%nyquist_velo(1)=dble(inp_nyquist_velo(1))
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: nyquist_velocity'
Goto 999
End If
!
Case('lat')
status=nf90_get_var(ncid,iVar,inp_latitude)
rad_syst%latitude(1)=dble(inp_latitude)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: lat'
Goto 999
End If
!
Case('lon')
status=nf90_get_var(ncid,iVar,inp_longitude)
rad_syst%longitude(1)=dble(inp_longitude)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: lon'
Goto 999
End If
!
Case('alt')
status=nf90_get_var(ncid,iVar,inp_altitude_sea_level)
rad_syst%altitude_sea_level(1)=dble(inp_altitude_sea_level)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: alt'
Goto 999
End If
!
Case('altitude_agl')
status=nf90_get_var(ncid,iVar,inp_altitude_from_ground)
rad_syst%altitude_from_ground(1)=inp_altitude_from_ground
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: altitude_agl'
Goto 999
End If
!


!Case default
!Call write_info('Note: variable present in NetCDF file but not read: &
!               & '//Trim(Adjustl(name)))

End Select
!
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var:'
Goto 999
End If
!
End Do
!
!!
!! input data managment:
raw_data%numb_scans(:)=raw_data%numb_scans(:)+1 ! change of Sweep number,0-based to 1-based
raw_data%sweep_start_ray_index(:)=raw_data%sweep_start_ray_index(:)+1 ! change of 0-based to 1-based
raw_data%sweep_end_ray_index(:)=raw_data%sweep_end_ray_index(:)+1     ! change of 0-based to 1-based
raw_data%time_s(:)=raw_data%base_time(1)+raw_data%offset_time(:) ! offset time in second since  1970-1-1 0:00:00 0:00
!!
if (sacr_param%mode=='vptmo') Then
      if (raw_data%nsweep==1) then
          if (raw_data%sweep_start_ray_index(1)/=1) then
              raw_data%sweep_start_ray_index(1)=1    
              call write_warning('Input var. sweep_start_ray_index redefined.Check the input file.')
          endif
          if (raw_data%sweep_end_ray_index(1)/=nt) then
              call write_warning('Input var. sweep_start_ray_index redefined.Check the input file.')
              raw_data%sweep_end_ray_index(1)=nt
          end if
       endif
endif
!
!! frequency from Hz to GHz
rad_syst%frequency=dble(inp_frequency*1.e-9)
!!
!do it=1,raw_data%nsweep
!w1=int(raw_data%sweep_start_ray_index(it))
!w2=int(raw_data%sweep_end_ray_index(it))
!write(*,*) 'old',it,w1,w2,w2-w1,real(fixed_angle(it))
!write(*,*) 'el:', minval(inp_elevation(w1:w2)),maxval(inp_elevation(w1:w2))
!write(*,*) 'az:', minval(inp_azimuth(w1:w2)),maxval(inp_azimuth(w1:w2))
!write(*,*) '   '
!enddo
!write(*,*) '--------------'
!!
!! correction for the sweep indices
If (sacr_param%mode/='vptmo') Then
!!
If (sacr_param%csweep==1) Then

modID=0
if (sacr_param%mode=='ppimo') modID=1
!
Allocate(inp_azimuth1(nt))
inp_azimuth1=inp_azimuth
if ((sacr_param%mode/='ppimo')) Then
where(nint(inp_azimuth1)==0) inp_azimuth=360.e0
do it=2,nt-1

d1=abs(inp_azimuth(it)-inp_azimuth(it-1))
d2=abs(inp_azimuth(it+1)-inp_azimuth(it))
d3=abs(inp_azimuth(it+1)-inp_azimuth(it-1))

if ((d3<d1) .and. (d3<d2)) then
      inp_azimuth(it)=inp_azimuth(it-1)+0.5*(inp_azimuth(it+1)-inp_azimuth(it-1))
end if
enddo
End if ! if (((sacr_param%mode/='ppimo')) 

Allocate(awork(nt)); awork=-999.e0
Allocate(ij1(nt),ij2(nt)) ; ij1=-1; ij2=-1
call CorrectSweepIndices(modID,nt,dble(inp_azimuth),dble(inp_elevation),nsweep,awork,ij1,ij2)
!
    if (raw_data%nsweep/=nsweep) Then
        call write_warning('Number of sweeps different than in input file.')
        Deallocate(raw_data%numb_scans,raw_data%sweep_start_ray_index,raw_data%sweep_end_ray_index)
        Deallocate(fixed_angle)
        raw_data%nsweep=nsweep
        Allocate(raw_data%numb_scans(nsweep),raw_data%sweep_start_ray_index(nsweep),raw_data%sweep_end_ray_index(nsweep))
        Allocate(fixed_angle(nsweep))
        !
        raw_data%sweep_start_ray_index=-999
        raw_data%sweep_end_ray_index=-999
        fixed_angle=-999.e0
        !
        do it=1,nsweep
        raw_data%numb_scans(it)=it
        fixed_angle(it)=real(awork(it))
        raw_data%sweep_start_ray_index(it)=ij1(it)
        raw_data%sweep_end_ray_index(it)=ij2(it)
        enddo

    else
        do it=1,nsweep
        fixed_angle(it)=real(awork(it))
        raw_data%sweep_start_ray_index(it)=ij1(it)
        raw_data%sweep_end_ray_index(it)=ij2(it)
        enddo
    endif
!---------------------------------------------------------------------- 
!do it=1,nsweep
!write(*,*) 'new:',it,raw_data%sweep_start_ray_index(it),raw_data%sweep_end_ray_index(it),raw_data%sweep_end_ray_index(it)-raw_data%sweep_start_ray_index(it),real(fixed_angle(it))
!write(*,*) 'el:',real(inp_elevation(raw_data%sweep_start_ray_index(it))),real(inp_elevation(raw_data%sweep_end_ray_index(it)))
!write(*,*) 'az:',real(inp_azimuth(raw_data%sweep_start_ray_index(it))),real(inp_azimuth(raw_data%sweep_end_ray_index(it)))
!write(*,*) '   '
!enddo
!
Deallocate(awork,ij1,ij2)
!
EndIf ! if (sacr_param%csweep==1)
!!
EndIf ! If (sacr_param%mode/='vptmo') Then
!!
!! azimuth change 
!! in input arm files North is zero degrees. Convention here: East is zero degrees.
do it=1,nt
if (inp_azimuth(it)==360) inp_azimuth(it)=0.e0
if ((inp_azimuth(it)<0) .and. (nint(inp_azimuth(it))==0)) inp_azimuth(it)=0.e0
!
if ((inp_azimuth(it)>=0).and.(inp_azimuth(it)<=90)) Then
raw_data%azimuth(it)=90.d0-dble(inp_azimuth(it))
else
raw_data%azimuth(it)=360.d0-(dble(inp_azimuth(it))-90.d0)
endif
enddo
!!
!!
Deallocate(inp_offset_time,inp_range,inp_azimuth,inp_elevation, &
         inp_DBZ,inp_VELO,inp_SNR, inp_LDR, inp_WIDTH, &
         inp_numb_scans,inp_sweep_start_ray_index,inp_sweep_end_ray_index,&
         inp_frequency,inp_nyquist_velo)
!!
Deallocate(fixed_angle)
!!
!
999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=nf90_close(ncid)
Endif
!
!
return
end subroutine ReadInpNCDF_var_raw_data
!!
!!
subroutine read_sacr_noise_data(InpFile,noise)
use radar_mods
Implicit None
!
Character(len=*),Intent(In)        :: InpFile
Type(sacr_noise), Intent(InOut)    :: noise
!
integer                            :: i
integer,parameter                  :: id=1
!
open(unit=id,file=InpFile)
do i=1,noise%nn
read(id,*) noise%n_mean(i), noise%n_std(i)
enddo
close(id)
!
! assign the elevations for the noise data
do i=1,noise%nn
noise%elevation(i)=dble(i)
enddo
!
return
end subroutine read_sacr_noise_data

