!! ----------------------------------------------------------------------------
!! *PROGRAM* WriteOutNCDF
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/WriteOutNCDF.f90
!!
!!
!! *LAST CHANGES*
!!
!!
!!  -May 16, 2014: A.T. Program WriteOutNCDF.f90 modified:
!!                          - The order of written output variables is changed
!!                          - Missing value for all variables changed from -999.e0 to -999.d0
!!                          - the configuration parametres figuring in the script added in the output ncdf file
!!                          - Added new output variables mergedsonde_flag,sounde_flag, ceilo_flag indicating
!!                            if the corresponding data were available (=1) or not (=0) for the run
!!                          - Global atributes include the name and version of the software
!!                          - The output data structure made dependent of the configuration parametrs,
!!                            for example: if mergedsonde_flag=0 and sounde_flag=0, the  variables
!!                            not present in the output file are all soundingvariables and all variables related
!!                            to velocity folding and attenuation correction.See User Guide for more details.
!!                          - Added new output variable -time interval between the scan time  and the closest sounding
!!
!!
!!
!! *DESCRIPTION* 
!!
!! This program contains subroutine for writing the output netcdf data files
!! 
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
!!
subroutine write_ncdf_output(FileName,raw_data_per_scan,folding,rad_syst,sacr_param,sounding,ceilo_data,status)
Use netcdf
Use write_messages
use radar_mods
Implicit None
!
Character(len=*),Intent(In)                       :: FileName
Type(sacr_raw_data_per_scan),Intent(In)           :: raw_data_per_scan
Type(sacr_folding_vars),Intent(In)                :: folding
Type(sacr_system_constants),Intent(In)            :: rad_syst
Type(sacr_parameters),Intent(In)                  :: sacr_param 
Type(sacr_sounding_data),Intent(In)               :: sounding
Type(sacr_ceilo_data),Intent(In)                  :: ceilo_data
integer,Intent(Out)                               :: status
!
integer                                 :: ncid
Character(len=nf90_max_name)            :: err_msg
!
! dimensions
integer                                 :: maxCells   ! number of range gates
integer                                 :: time       ! number of elevations
integer                                 :: numSystems ! =1
integer                                 :: sounde     ! number of sounding data

!dimIDs 
integer                                 :: maxCellsID,timeID,numSystemsID,soundeID
!varIDs
integer                                 :: RangeID,base_timeID,time_offsetID,radar_timeID,&
                                           Fixed_AngleID,Range_to_First_CellID,radar_frequencyID
integer                                 :: Nyquist_VelocityID,LatitudeID,LongitudeID,AltitudeID,Altitude_fglID
integer                                 :: AzimuthID,ElevationID,DBZID,VRID,SWID,SNRID,DBZ_corID
integer                                 :: aliasnID,wfID,wdID,Up_90ID,Up_azID,Up_az_elID
integer                                 :: VD_unfoldedID,VD_unf_flagID,percentage_flagID
integer                                 :: elevation_correctedID,Range_correctedID,&
                                           x_distanceID,z_distanceID
integer                                 :: LDRID,DBZ_received_powerID
integer                                 :: dt_sounding_scanID
integer                                 :: TemperatureID,heightID,wspd_soundingID,wdir_soundingID
!
integer                                 :: sounde_flagID,msounde_flagID,ceilo_flagID
integer                                 :: csweep_flagID, ir_flagID,ir_ldr_threshID
integer                                 :: ste_flagID,ste_el_minID,ste_el_maxID,ste_vel_threshID
integer                                 :: M1_flagID,M2_flagID,M3_flagID,M4_flagID
!
!
Character(len=nf90_max_name)            :: att_string
real*8,Dimension(:),Allocatable         :: work1d
integer,Dimension(:),Allocatable        :: work1i 
!
maxCells   = raw_data_per_scan%ngates
time       = raw_data_per_scan%ntimes
numSystems = raw_data_per_scan%one
sounde     = sounding%sounde
!
!!
   !  create file
   status =  nf90_create(FileName, NF90_CLOBBER, ncid)
     If (status /= 0) then ; err_msg='Error in nf90_create' ; Goto 999 ; endif
   ! define dimensions
   status = nf90_def_dim(ncid, "maxCells",maxCells,maxCellsID)
     If (status /= 0) then ; err_msg='Error in nf90_def_dim: maxCells' ;  Goto 999 ; endif
   status = nf90_def_dim(ncid, "time",time,timeID)
     If (status /= 0) then ; err_msg='Error in nf90_def_dim: time' ;  Goto 999 ; endif
   status = nf90_def_dim(ncid, "numSystems",numSystems,numSystemsID)
     If (status /= 0) then ; err_msg='Error in nf90_def_dim: numSystems' ;  Goto 999 ; endif

IF ((sounding%soundeID==1).or. (sounding%msoundeID==1)) THEN
   status = nf90_def_dim(ncid, "sounde",sounde,soundeID)
     If (status /= 0) then ; err_msg='Error in nf90_def_dim: sounde' ;  Goto 999 ; endif
ENDIF

   !  define variables
   !
   status = nf90_def_var(ncid, "m_time", nf90_double, (/numSystemsID/),base_timeID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:base_time' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "scan_time", nf90_double, (/timeID/), time_offsetID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:time_offset' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "radar_time", nf90_double, (/timeID/),radar_timeID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:radar_time' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Fixed_Angle", nf90_float, (/numSystemsID/),Fixed_AngleID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Fixed_Angle' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "radar_frequency", nf90_float, (/numSystemsID/),radar_frequencyID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:radar_frequency' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Nyquist_Velocity", nf90_float, (/numSystemsID/),Nyquist_VelocityID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Nyquist_Velocity' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Latitude", nf90_float, (/numSystemsID/),LatitudeID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Latitude' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Longitude", nf90_float, (/numSystemsID/),LongitudeID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Longitude' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Altitude", nf90_float, (/numSystemsID/),AltitudeID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Altitude' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Altitude_fgl", nf90_float, (/numSystemsID/),Altitude_fglID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Altitude_fgl' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Range_to_First_Cell", nf90_float,(/numSystemsID/),Range_to_First_CellID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Range_to_First_Cell' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Range", nf90_float, (/maxCellsID/), RangeID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var: Range' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Azimuth", nf90_float, (/timeID/),AzimuthID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Azimuth' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Elevation", nf90_float, (/timeID/),ElevationID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Elevation' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "DBZ", nf90_int, (/maxCellsID,timeID/),DBZID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:DBZ' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "VR", nf90_int, (/maxCellsID,timeID/),VRID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:VR' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "SW", nf90_int, (/maxCellsID,timeID/),SWID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:SW' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "SNR", nf90_int, (/maxCellsID,timeID/),SNRID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:SNR' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "elevation_corrected", nf90_float,(/maxCellsID,timeID/),elevation_correctedID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:elevation_corrected' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Range_corrected", nf90_float,(/maxCellsID,timeID/),Range_correctedID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Range_corrected' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "x_distance", nf90_float,(/maxCellsID,timeID/),x_distanceID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:x_distance' ;Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "z_distance", nf90_float,(/maxCellsID,timeID/),z_distanceID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:z_distance' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "LDR", nf90_int, (/maxCellsID,timeID/),LDRID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:LDR' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "DBZ_received_power", nf90_int,(/maxCellsID,timeID/),DBZ_received_powerID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:DBZ_received_power' ; Goto 999 ; endif
   !

IF ((sounding%soundeID==1).or.(sounding%msoundeID==1)) THEN
   status = nf90_def_var(ncid, "DBZ_cor", nf90_int, (/maxCellsID,timeID/),DBZ_corID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:DBZ_cor' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "VD_unfolded", nf90_int, (/maxCellsID,timeID/),VD_unfoldedID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Up_az_el' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "VD_unf_flag", nf90_int,(/timeID/),VD_unf_flagID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:VD_unf_flag' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "aliasn", nf90_int,(/maxCellsID,timeID/),aliasnID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:aliasn' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "percentage_flag", nf90_float,(/numSystemsID/),percentage_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:percentage_flag' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Up_90", nf90_int, (/maxCellsID,timeID/),Up_90ID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Up_90' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Up_az", nf90_int, (/maxCellsID,timeID/),Up_azID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Up_az' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Up_az_el", nf90_int,(/maxCellsID,timeID/),Up_az_elID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Up_az_el' ; Goto 999 ; endif
   !

   status = nf90_def_var(ncid, "wf", nf90_int, (/maxCellsID,timeID/),wfID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:wf' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "wd", nf90_int, (/maxCellsID,timeID/),wdID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:wd' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "dt_sounding_scan",nf90_float,(/numSystemsID/),dt_sounding_scanID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var: dt_sounding_scan'; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "Temperature", nf90_float, (/soundeID/),TemperatureID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:Temperature' ; Goto 999 ; endif
   status = nf90_def_var(ncid, "height", nf90_float, (/soundeID/),heightID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:height' ; Goto 999 ; endif
   status = nf90_def_var(ncid, "wspd_sounding", nf90_float, (/soundeID/),wspd_soundingID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:wspd_sounding' ; Goto 999 ; endif
   status = nf90_def_var(ncid, "wdir_sounding", nf90_float, (/soundeID/),wdir_soundingID )
            If (status /= 0) then ; err_msg='Error in nf90_def_var:wdir_sounding' ; Goto 999 ; endif

ENDIF

!!!
   status = nf90_def_var(ncid,"mergedsonde_flag",nf90_int,(/numSystemsID/),msounde_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:msounde_flag'; Goto 999 ; endif
   !
   status = nf90_def_var(ncid, "sounde_flag",nf90_int,(/numSystemsID/),sounde_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:sounde_flag' ; Goto 999 ; endif
   !
   status = nf90_def_var(ncid,"ceilo_flag",nf90_int,(/numSystemsID/),ceilo_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:ceilo_flag' ; Goto 999 ; endif
   !
if ((Trim(sacr_param%mode)/='vptmo')) then
   status = nf90_def_var(ncid,"csweep_flag",nf90_int,(/numSystemsID/),csweep_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:csweep_flag'; Goto 999 ; endif
end if 
   !
If ((ceilo_data%ceiloID==0).and.(sounding%soundeID==0).and.(sounding%msoundeID==0)) Then
   go to 345
Else
   status = nf90_def_var(ncid,"ir_flag",nf90_int,(/numSystemsID/),ir_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:ir_flag';Goto 999 ; endif
   !!
   IF (sacr_param%irID==1) Then
      status = nf90_def_var(ncid,"ir_ldr_thresh",nf90_float,(/numSystemsID/),ir_ldr_threshID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:ir_ldr_thresh'; Goto 999 ; endif
   ENDIF
EndIf
345 continue
   !!
if (Trim(sacr_param%mode)/='vptmo') then
   status =nf90_def_var(ncid,"ste_flag",nf90_int,(/numSystemsID/),ste_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:ste_flag'; Goto 999 ; endif
   !!
   IF (sacr_param%steID==1) Then
   status = nf90_def_var(ncid,"ste_el_min",nf90_float,(/numSystemsID/),ste_el_minID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:ste_el_min'; Goto 999 ; endif
   !
   status =nf90_def_var(ncid,"ste_el_max",nf90_float,(/numSystemsID/),ste_el_maxID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:ste_el_max'; Goto 999 ; endif 
   !
   status = nf90_def_var(ncid,"ste_vel_thresh",nf90_float,(/numSystemsID/),ste_vel_threshID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:ste_vel_thresh'; Goto 999 ; endif
   END IF
endif ! if (Trim(sacr_param%mode)/='vptmo')
!!
!!
   status = nf90_def_var(ncid,"M1_flag",nf90_int,(/numSystemsID/),M1_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:M1_flag'; Goto 999 ; endif
   !
   status = nf90_def_var(ncid,"M2_flag",nf90_int,(/numSystemsID/),M2_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:M2_flag'; Goto 999 ; endif
   !
   status = nf90_def_var(ncid,"M3_flag",nf90_int,(/numSystemsID/),M3_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:M3_flag'; Goto 999 ; endif
   !
  status = nf90_def_var(ncid,"M4_flag",nf90_int,(/numSystemsID/),M4_flagID)
            If (status /= 0) then ; err_msg='Error in nf90_def_var:M4_flag'; Goto 999 ; endif
   !
   !
   ! Attributes
   !
   !!
    status = nf90_put_att(ncid,base_timeID,'units','seconds since 1970-0101 00:00 UTC')
!
    status = nf90_put_att(ncid,time_offsetID,'description','Unix Date/Time value for every record')
    status = nf90_put_att(ncid,time_offsetID,'units','seconds since 1970-0101 00:00 UTC')
!
    status = nf90_put_att(ncid,radar_timeID,'description','Time from midnight for every record')
    status = nf90_put_att(ncid,radar_timeID,'units','hour') 
!
    status = nf90_put_att(ncid,Fixed_AngleID,'description','Targeted fixed angle for this scan')
    status = nf90_put_att(ncid,Fixed_AngleID,'units','degrees')
!
    status = nf90_put_att(ncid,radar_frequencyID,'description','Radar frequency')
    status = nf90_put_att(ncid,radar_frequencyID,'units','GigaHertz')
!
    status = nf90_put_att(ncid,Nyquist_VelocityID,'description','Unambiguous Doppler velocity')
    status = nf90_put_att(ncid,Nyquist_VelocityID,'units','meters per second')
!
    status = nf90_put_att(ncid,LatitudeID,'description','Latitude of the instrument')
    status = nf90_put_att(ncid,LatitudeID,'units','degrees north')
!
    status = nf90_put_att(ncid,LongitudeID,'description','Longitude of the instrument')
    status = nf90_put_att(ncid,LongitudeID,'units','degrees east')
!
    status = nf90_put_att(ncid,AltitudeID,'description','Altitude above mean sea level')
    status = nf90_put_att(ncid,AltitudeID,'units','meters')
!
    status = nf90_put_att(ncid,Altitude_fglID,'description','Altitude of radar instrument from the ground level')
    status = nf90_put_att(ncid,Altitude_fglID,'units','meters')
!
    status = nf90_put_att(ncid,Range_to_First_CellID,'description','Range to the center of the first cell')
    status = nf90_put_att(ncid,Range_to_First_CellID,'units','meters')
!
    status = nf90_put_att(ncid,RangeID,'description','Range to measurement volume')
    status = nf90_put_att(ncid,RangeID,'units','km')
!
    status = nf90_put_att(ncid,AzimuthID,'description','Azimuth angle from true east')
    status = nf90_put_att(ncid,AzimuthID,'units','degrees counterclockwise from true east')
!
    status = nf90_put_att(ncid,ElevationID,'description','Elevation angle from horizontal')
    status = nf90_put_att(ncid,ElevationID,'units','degrees')
!
    status = nf90_put_att(ncid,DBZID,'description','Equivalent reflectivity factor')
    status = nf90_put_att(ncid,DBZID,'units','dBZ')
    status = nf90_put_att(ncid,DBZID,'scale_factor',real(rad_syst%dbz_fact))
    status = nf90_put_att(ncid,DBZID,'add_offset',real(rad_syst%dbz_offset))
    status = nf90_put_att(ncid,DBZID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,DBZID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,VRID,'description','Radial mean Doppler velocity of scatterers (negative towards the instrument')
    status = nf90_put_att(ncid,VRID,'units','meters per second')
    status = nf90_put_att(ncid,VRID,'scale_factor',real(rad_syst%velo_fact))
    status = nf90_put_att(ncid,VRID,'add_offset',real(rad_syst%velo_offset))
    status = nf90_put_att(ncid,VRID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,VRID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,SWID,'description','Spectrum width')
    status = nf90_put_att(ncid,SWID,'units','meters per second')
    status = nf90_put_att(ncid,SWID,'scale_factor',real(rad_syst%sw_fact))
    status = nf90_put_att(ncid,SWID,'add_offset',real(rad_syst%sw_offset))
    status = nf90_put_att(ncid,SWID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,SWID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,SNRID,'description','Signal to Noise Ratio')
    status = nf90_put_att(ncid,SNRID,'units','dB')
    status = nf90_put_att(ncid,SNRID,'scale_factor',real(rad_syst%snr_fact))
    status = nf90_put_att(ncid,SNRID,'add_offset',real(rad_syst%snr_offset))
    status = nf90_put_att(ncid,SNRID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,SNRID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,elevation_correctedID,'description','Elevations corrected from curvature')
    status = nf90_put_att(ncid,elevation_correctedID,'units','degrees')
    status = nf90_put_att(ncid,elevation_correctedID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,Range_correctedID,'description','Ranges correctedfrom curvature')
    status = nf90_put_att(ncid,Range_correctedID,'units','km')
    status = nf90_put_att(ncid,Range_correctedID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,x_distanceID,'description','X_distances corrected from curvature')
    status = nf90_put_att(ncid,x_distanceID,'units','km')
    status = nf90_put_att(ncid,x_distanceID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,z_distanceID,'description','Z_distances corrected from curvature')
    status = nf90_put_att(ncid,z_distanceID,'units','km')
    status = nf90_put_att(ncid,z_distanceID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,LDRID,'description','Linear depolarization ratio')
    status = nf90_put_att(ncid,LDRID,'units','dB')
    status = nf90_put_att(ncid,LDRID,'scale_factor',real(rad_syst%ldr_fact))
    status = nf90_put_att(ncid,LDRID,'add_offset',real(rad_syst%ldr_offset))
    status = nf90_put_att(ncid,LDRID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,LDRID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,DBZ_received_powerID,'description','DBZ received power')
    status = nf90_put_att(ncid,DBZ_received_powerID,'units','dBZ')
    status = nf90_put_att(ncid,DBZ_received_powerID,'scale_factor',real(rad_syst%dbz_fact))
    status = nf90_put_att(ncid,DBZ_received_powerID,'add_offset',real(rad_syst%dbz_offset))
    status = nf90_put_att(ncid,DBZ_received_powerID,'comment','To unpack field,multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,DBZ_received_powerID,'missing_value','-999.d0')
!
IF ((sounding%soundeID==1).or.(sounding%msoundeID==1)) THEN

    status = nf90_put_att(ncid,DBZ_corID,'description','Reflectivity corrected for gaseous attenuation')
    status = nf90_put_att(ncid,DBZ_corID,'units','dBZ')
    status = nf90_put_att(ncid,DBZ_corID,'scale_factor',real(rad_syst%dbz_fact))
    status = nf90_put_att(ncid,DBZ_corID,'add_offset',real(rad_syst%dbz_offset))
    status = nf90_put_att(ncid,DBZ_corID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,DBZ_corID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,VD_unfoldedID,'description','Unfolded Doppler Velocity,negative towards the instrument')
    status = nf90_put_att(ncid,VD_unfoldedID,'units','meters per second')
    status = nf90_put_att(ncid,VD_unfoldedID,'scale_factor',real(rad_syst%velo_fact))
    status = nf90_put_att(ncid,VD_unfoldedID,'add_offset',real(rad_syst%velo_offset))
    status = nf90_put_att(ncid,VD_unfoldedID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,VD_unfoldedID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,VD_unf_flagID,'description','de-aliasing quality control (=0 correction failed, =1 correction works) ')
    status = nf90_put_att(ncid,VD_unf_flagID,'units','-')
    status = nf90_put_att(ncid,VD_unf_flagID,'missing_value','-999')
    status = nf90_put_att(ncid,VD_unf_flagID,'comment','if flag=1 indicating that correction works,and visually it may be seen oposit, likelly  Number of aliasing estimated from sounding wind is uncertain')
!
    status = nf90_put_att(ncid,aliasnID,'description','Number of aliasing')
    status = nf90_put_att(ncid,aliasnID,'units','-')
    status = nf90_put_att(ncid,aliasnID,'missing_value','-999')
!
    status = nf90_put_att(ncid,percentage_flagID,'description','100 x (number of rays where folding correction failed /number of rays with cloud)')
    status = nf90_put_att(ncid,percentage_flagID,'units','%')
    status = nf90_put_att(ncid,percentage_flagID,'missing_value','-999')
!
    status = nf90_put_att(ncid,Up_90ID,'description','Wind speed from sounding projected 90 degrees to the radar scan')
    status = nf90_put_att(ncid,Up_90ID,'units','meters per second')
    status = nf90_put_att(ncid,Up_90ID,'scale_factor',real(rad_syst%velo_fact))
    status = nf90_put_att(ncid,Up_90ID,'add_offset',real(rad_syst%velo_offset))
    status = nf90_put_att(ncid,Up_90ID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,Up_90ID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,Up_azID,'description','Wind speed from sounding projected to the azimuth of radar scan')
    status = nf90_put_att(ncid,Up_azID,'units','meters per second')
    status = nf90_put_att(ncid,Up_azID,'scale_factor',real(rad_syst%velo_fact))
    status = nf90_put_att(ncid,Up_azID,'add_offset',real(rad_syst%velo_offset))
    status = nf90_put_att(ncid,Up_azID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,Up_azID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,Up_az_elID,'description','Wind speed from sounding projected to the azimuth and elevation of radar scan')
    status = nf90_put_att(ncid,Up_az_elID,'units','meters per second')
    status = nf90_put_att(ncid,Up_az_elID,'scale_factor',real(rad_syst%velo_fact))
    status = nf90_put_att(ncid,Up_az_elID,'add_offset',real(rad_syst%velo_offset))
    status = nf90_put_att(ncid,Up_az_elID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,Up_az_elID,'missing_value','-999.d0')
!

    status = nf90_put_att(ncid,wfID,'description','wind force')
    status = nf90_put_att(ncid,wfID,'units','meters per second')
    status = nf90_put_att(ncid,wfID,'scale_factor',real(rad_syst%velo_fact))
    status = nf90_put_att(ncid,wfID,'add_offset',real(rad_syst%velo_offset))
    status = nf90_put_att(ncid,wfID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,wfID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,wdID,'description','wind direction from true east')
    status = nf90_put_att(ncid,wdID,'units','degrees counterclockwise from true east')
    status = nf90_put_att(ncid,wdID,'scale_factor',real(rad_syst%velo_fact))
    status = nf90_put_att(ncid,wdID,'add_offset',real(rad_syst%velo_offset))
    status = nf90_put_att(ncid,wdID,'comment','To unpack field, multiply values by the scale_factor attribute, then add the add_offset attribute.')
    status = nf90_put_att(ncid,wdID,'missing_value','-999.d0')
!

    status = nf90_put_att(ncid,dt_sounding_scanID,'description','time interval between sounding time and sacr scan time')
    status = nf90_put_att(ncid,dt_sounding_scanID,'units','sec')
    status = nf90_put_att(ncid,dt_sounding_scanID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,TemperatureID,'description','Temperature from sounding')
    status = nf90_put_att(ncid,TemperatureID,'units','Celsius')
    status = nf90_put_att(ncid,TemperatureID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,heightID,'description','heigh from sounding (above MSL)')
    status = nf90_put_att(ncid,heightID,'units','km')
    status = nf90_put_att(ncid,heightID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,wspd_soundingID,'description','wind speed from sounding')
    status = nf90_put_att(ncid,wspd_soundingID,'units','m/s')
    status = nf90_put_att(ncid,wspd_soundingID,'missing_value','-999.d0')
!
    status = nf90_put_att(ncid,wdir_soundingID,'description','wind direction from sounding')
    status = nf90_put_att(ncid,wdir_soundingID,'units','degrees from East')
    status = nf90_put_att(ncid,wdir_soundingID,'missing_value','-999.d0')
!
ENDIF

!-------------------------------------------------------------------------------------------
!
    status = nf90_put_att(ncid,msounde_flagID,'description','=1  mergedsonde data available, =0 mergedsonde data not available')
    status = nf90_put_att(ncid,msounde_flagID,'units','-')
!
    status = nf90_put_att(ncid,sounde_flagID,'description','=1 sounde data available, =0 sounde data not available')
    status = nf90_put_att(ncid,sounde_flagID,'units','-')
!
    status = nf90_put_att(ncid,ceilo_flagID,'description','=1 ceilometer data available, =0 ceilometer data not available')
    status = nf90_put_att(ncid,ceilo_flagID,'units','-')
!
if ((Trim(sacr_param%mode)/='vptmo')) then
    status = nf90_put_att(ncid,csweep_flagID,'description','correction for sweep indices is applied (=1) or not (=0)')
    status = nf90_put_att(ncid,csweep_flagID,'units','-')
endif
!
If ((ceilo_data%ceiloID==0).and.(sounding%soundeID==0).and.(sounding%msoundeID==0)) Then
go to 346
Else
!
    status = nf90_put_att(ncid,ir_flagID,'description',' correction for the insect removal is applied (=1) or not (=0) ')
    status = nf90_put_att(ncid,ir_flagID,'units','-')
!
    IF (sacr_param%irID==1) Then
       status = nf90_put_att(ncid,ir_ldr_threshID,'description','LDR treshold for the insect removal')
       status = nf90_put_att(ncid,ir_ldr_threshID,'units','dB')
    ENDIF
EndIf
346 continue
    !
if (Trim(sacr_param%mode)/='vptmo') then
    status = nf90_put_att(ncid,ste_flagID,'description','correction for the second trip echo removal is applied (=1) or is not (=0)')
    status = nf90_put_att(ncid,ste_flagID,'units','-')
!
IF (sacr_param%steID==1) Then
    status = nf90_put_att(ncid,ste_el_minID,'description','elevation value below which the second trip echo occurence is checked')
    status = nf90_put_att(ncid,ste_el_minID,'units','degrees counterclockwise from true east')
    !
    status = nf90_put_att(ncid,ste_el_maxID,'description','elevation value above which the second trip echo occurence is checked')
    status = nf90_put_att(ncid,ste_el_maxID,'units','degrees counterclockwise from true east')
    !
    status = nf90_put_att(ncid,ste_vel_threshID,'description','treshold value of velocity standard deviation used for identification of second trip echo')
    status = nf90_put_att(ncid,ste_vel_threshID,'units','m/s')
    !
ENDIF
endif ! if (Trim(sacr_param%mode)/='vptmo')
!
    status = nf90_put_att(ncid,M1_flagID,'description','MASK 1 (removing noise pixels using H&S74 and/or climat.noise values) is applied (=1) or not (=0)')
    status = nf90_put_att(ncid,M1_flagID,'units','-')
!
    status = nf90_put_att(ncid,M2_flagID,'description','MASK 2 (removing false detection streaks along radial using ldr ) is applied (=1) or not (=0)')
    status = nf90_put_att(ncid,M2_flagID,'units','-') 
!
    status = nf90_put_att(ncid,M3_flagID,'description','MASK 3 (using i-box filter) is applied (=1) or not (=0)')
    status = nf90_put_att(ncid,M3_flagID,'units','-')
!
    status = nf90_put_att(ncid,M4_flagID,'description','MASK 4 ( final masking at the end of processing) is applied (=1) or not (=0)')
    status = nf90_put_att(ncid,M4_flagID,'units','-')
!
! ------------------------------------------------------------------------------
!!!
!--------------
!  Global Attributes
   status = nf90_put_att(ncid, NF90_GLOBAL,'content','This file contains one scan of remotely sensed data')
   att_string='Scanning Remote Sensor at '//Trim(sacr_param%location)//' site'
   status = nf90_put_att(ncid, NF90_GLOBAL,'conventions',att_string)
   att_string=Trim(sacr_param%radar)
   status = nf90_put_att(ncid, NF90_GLOBAL,"Instrument_Name",att_string)
   status = nf90_put_att(ncid, NF90_GLOBAL,'Instrument_Type','ground')
   att_string=Trim(sacr_param%mode)
   status = nf90_put_att(ncid, NF90_GLOBAL,'Scan_Mode',att_string)
   att_string=Trim(raw_data_per_scan%time_str)
   status = nf90_put_att(ncid, NF90_GLOBAL,'Time_at_the_middle_of_the_scan',att_string)
   att_string='Netcdf File for the '//Trim(sacr_param%radar)//' mode '//Trim(sacr_param%mode)
   status = nf90_put_att(ncid, NF90_GLOBAL,'description',att_string)
   att_string=Trim(rad_syst%rfilename)
   status = nf90_put_att(ncid, NF90_GLOBAL,'input_radar_file',att_string)
!
   att_string=Trim(rad_syst%sfilename)
   status = nf90_put_att(ncid, NF90_GLOBAL,'input_sounding_file',att_string)
!!
   att_string=Trim(rad_syst%cfilename)
   status = nf90_put_att(ncid, NF90_GLOBAL,'input_ceilometer_file',att_string)
   status = nf90_put_att(ncid, NF90_GLOBAL,'created_by','sacr_v1.1')
   status = nf90_put_att(ncid, NF90_GLOBAL,'author','http://wwww.clouds.mcgill.ca')
   status = nf90_put_att(ncid, NF90_GLOBAL,'institute','http://wwww.clouds.mcgill.ca')
   

   ! Close the definition part
   status = nf90_enddef(ncid)
            If (status /= 0) then ; err_msg='Error in nf90_enddef' ;  Goto 999 ; endif
!!
!!
   !
   ! write variables
   !
   status =  nf90_put_var(ncid, base_timeID,raw_data_per_scan%time_s(raw_data_per_scan%time_str_index))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :base_time ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, time_offsetID, raw_data_per_scan%time_s)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :time_offset ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, radar_timeID, raw_data_per_scan%radar_time)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :radar_time ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, Fixed_AngleID,real(raw_data_per_scan%fixed_angle))
      If (status /= 0) then ; err_msg='Error in nf90_put_var : Fixed_Angle' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, radar_frequencyID,real(rad_syst%frequency))
      If (status /= 0) then ; err_msg='Error in nf90_put_var : radar_frequency' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, Nyquist_VelocityID, real(rad_syst%nyquist_velo))
      If (status /= 0) then ; err_msg='Error in nf90_put_var : Nyquist_Velocity' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, LatitudeID,real(rad_syst%latitude) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Latitude ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, LongitudeID,real(rad_syst%longitude) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Longitude ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, AltitudeID,real(rad_syst%altitude_sea_level))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Altitude ' ; Goto 999 ; endif
   !
    status =  nf90_put_var(ncid, Altitude_fglID,real(rad_syst%altitude_from_ground))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Altitude_from_ground ' ; Goto 999 ; endif
   !
   Allocate(work1d(numSystems))
   work1d=raw_data_per_scan%range(1)+0.5d0*(raw_data_per_scan%range(3)-raw_data_per_scan%range(2))
   status =  nf90_put_var(ncid, Range_to_First_CellID,real(work1d))
   Deallocate(work1d)
      If (status /= 0) then ; err_msg='Error in nf90_put_var : Range_to_First_Cell' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, RangeID,real(raw_data_per_scan%range*1.d-3))
      If (status /= 0) then ; err_msg='Error in nf90_put_var Range : ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, AzimuthID,real(raw_data_per_scan%azimuth) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Azimuth ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, ElevationID,real(raw_data_per_scan%elevation) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Elevation ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, DBZID,idint((raw_data_per_scan%DBZ-rad_syst%dbz_offset)/rad_syst%dbz_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :DBZ ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, VRID,idint((raw_data_per_scan%VELO-rad_syst%velo_offset)/rad_syst%velo_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :VR ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, SWID,idint((raw_data_per_scan%WIDTH-rad_syst%sw_offset)/rad_syst%sw_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :SW ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, SNRID,idint((raw_data_per_scan%SNR-rad_syst%snr_offset)/rad_syst%snr_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :SNR ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid,elevation_correctedID,real(raw_data_per_scan%elevation_corrected))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :elevation_corrected ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid,Range_correctedID,real(raw_data_per_scan%Range_corrected*1.d-3) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Range_corrected ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid,x_distanceID,real(raw_data_per_scan%x_distance*1.d-3))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :x_distance ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid,z_distanceID,real(raw_data_per_scan%z_distance*1.d-3) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :z_distance ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid,LDRID,idint((raw_data_per_scan%LDR-rad_syst%ldr_offset)/rad_syst%ldr_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :LDR ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid,DBZ_received_powerID,idint((raw_data_per_scan%DBZ_rc-rad_syst%dbz_offset)/rad_syst%dbz_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :DBZ_received_power' ; Goto 999 ; endif
   !
   !!
IF ((sounding%soundeID==1).or.(sounding%msoundeID==1)) THEN
   !!
   status =  nf90_put_var(ncid, DBZ_corID,idint((raw_data_per_scan%DBZ_cor-rad_syst%dbz_offset)/rad_syst%dbz_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :DBZ_cor ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, VD_unfoldedID,idint((folding%VD_unfolded-rad_syst%velo_offset)/rad_syst%velo_fact) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :VD_unfolded ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, VD_unf_flagID,folding%VD_unf_flag)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :VD_unf_flag' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, aliasnID,folding%aliasn )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :aliasn ' ; Goto 999 ; endif
   !
    status =  nf90_put_var(ncid, percentage_flagID,real(folding%percentage_flag))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :percentage_flag' ; Goto 999 ; endif
   !
   !
   status =  nf90_put_var(ncid,Up_90ID,idint((folding%Up_90-rad_syst%velo_offset)/rad_syst%velo_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Up_90 ' ; Goto 999; endif
   !
   status =  nf90_put_var(ncid,Up_azID,idint((folding%Up_az-rad_syst%velo_offset)/rad_syst%velo_fact))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Up_az ' ; Goto 999; endif
   !
   status =  nf90_put_var(ncid,Up_az_elID,idint((folding%Up_az_el-rad_syst%velo_offset)/rad_syst%velo_fact) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Up_az_el ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, wfID,dint((folding%wf-rad_syst%velo_offset)/rad_syst%velo_fact) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :wf ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, wdID,dint((folding%wd-rad_syst%velo_offset)/rad_syst%velo_fact) )
      If (status /= 0) then ; err_msg='Error in nf90_put_var :wd ' ; Goto 999 ; endif
   !
   Allocate(work1d(numSystems))
   work1d=sounding%dt_sounding_scan
   status =  nf90_put_var(ncid, dt_sounding_scanID,real(work1d))
      If (status /= 0) then ; err_msg='Error in nf90_put_var : dt_sounding_scan ' ; Goto 999 ; endif
   Deallocate(work1d)
   !
   status =  nf90_put_var(ncid, TemperatureID,real(sounding%Temperature))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :Temperature ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, heightID,real(sounding%altitude*1.d-3))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sounding altitude ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, wspd_soundingID,real(sounding%wf))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sounding  wf ' ; Goto 999 ; endif
   !
   status =  nf90_put_var(ncid, wdir_soundingID,real(sounding%wd))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sounding  wd ' ; Goto 999 ; endif
   !
ENDIF
!!
!-----------------------------------------------------------------------------------------------------

   Allocate(work1d(numSystems),work1i(numSystems))
   !
      work1i(1)=sounding%msoundeID
      status =  nf90_put_var(ncid, msounde_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sounding msoundeID'; Goto 999 ; endif
   !
      work1i(1)=sounding%soundeID
      status =  nf90_put_var(ncid, sounde_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sounding soundeID' ; Goto 999 ; endif
   !
      work1i(1)=ceilo_data%ceiloID
      status =  nf90_put_var(ncid, ceilo_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :ceilo data ceiloID'; Goto 999 ; endif
   !
if ((Trim(sacr_param%mode)/='vptmo')) then
      work1i(1)=sacr_param%csweep
      status =  nf90_put_var(ncid, csweep_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var : sacr_param csweep'; Goto 999 ; endif
endif
    !
If ((ceilo_data%ceiloID==0).and.(sounding%soundeID==0).and.(sounding%msoundeID==0)) Then
go to 347
Else
      work1i(1)=sacr_param%irID 
      status =  nf90_put_var(ncid, ir_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param irID' ; Goto 999 ; endif
   !
  IF (sacr_param%irID==1) Then
      work1d(1)=sacr_param%ir_ldr_thresh
      status =  nf90_put_var(ncid, ir_ldr_threshID,real(work1d))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param ir_ldr_thresh' ;Goto 999 ; endif
  END IF
EndIf
347 continue
   !
if (Trim(sacr_param%mode)/='vptmo') then
      work1i(1)=sacr_param%steID
      status =  nf90_put_var(ncid, ste_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param steID' ; Goto 999 ; endif
   !
IF (sacr_param%steID==1) Then
      !
      work1d(1)=sacr_param%ste_el_min
      status =  nf90_put_var(ncid, ste_el_minID,real(work1d))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param ste_el_min'; Goto 999 ; endif
      !
      work1d(1)=sacr_param%ste_el_max
      status =  nf90_put_var(ncid, ste_el_maxID,real(work1d))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param ste_el_max'; Goto 999 ; endif
      !
      work1d(1)=sacr_param%ste_vel_thresh
      status =  nf90_put_var(ncid,ste_vel_threshID,real(work1d))
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param ste_vel_thresh'; Goto 999 ; endif
      !
END IF
endif ! if (Trim(sacr_param%mode)/='vptmo'
   !
      work1i(1)=sacr_param%M1ID
      status =  nf90_put_var(ncid, M1_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param M1ID' ; Goto 999 ; endif
   !
      work1i(1)=sacr_param%M2ID
      status =  nf90_put_var(ncid, M2_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param M2ID' ; Goto 999 ; endif
   !
      work1i(1)=sacr_param%M3ID
      status =  nf90_put_var(ncid, M3_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param M3ID' ; Goto 999 ; endif
   !
      work1i(1)=sacr_param%fmID
      status =  nf90_put_var(ncid, M4_flagID,work1i)
      If (status /= 0) then ; err_msg='Error in nf90_put_var :sacr_param fmID' ; Goto 999 ; endif
   !
!-----------------------------------------------------------------------------------------------------
!-------------------------------------------------------------------------------------------
!
Deallocate(work1d,work1i)

   ! close the file
   status = nf90_close(ncid)
      If (status /= 0) then ; err_msg='Error in nf90_close' ;  Goto 999 ; endif
!!
999 if (status /= 0) then ; Call write_error(err_msg) ; Call Exit(1) ; endif
!
return
end subroutine write_ncdf_output
