!! ----------------------------------------------------------------------------
!! *PROGRAM* hroutines
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/hroutines.f90
!!
!!
!! *LAST CHANGES*
!!
!!
!!  -May 26, 2014: A.T.    Subroutines CorrectSweepIndices modified in order to                     
!!                         correctly estimate the number of actual scans, fixed angles,
!!                         and sweep indices, based solemnly on observed elevation and azimuth
!!                         values and input informaton wheather the fixed angle is azimuth or
!!                         elevation.
!!
!!  -May 23, 2014: A.T.    Removed subroutines get_numb_scans and correct_sweep_indices

!!
!!  -May 21-22, 2014: A.T. Written new subroutines CorrectSweepIndices and GetSweepIndices
!!                         that correctly estimate the number of actual scans,fixed angles,
!!                         and sweep indices, based solemnly on observed elevation and azimuth
!!                         values and the a-priori knowledge that the fixed angle is azimuth,
!!                         that is, for any mode different than "ppimo".
!!
!!
!! *DESCRIPTION* 
!!
!! This program contains the collection of subroutines that complement the post-processing 
!! algorithms in processing.f90. 
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------

subroutine get_3ch_string(ix,ix_str)
Implicit None
!
integer,intent(In)               :: ix
character(len=3),intent(InOut)   :: ix_str
!
if (ix.le.9)then
ix_str='00'
write(ix_str,'(a,(i1))') trim(ix_str),ix
else if((ix.ge.10) .and. (ix.le.99) ) then
ix_str='0'
write(ix_str,'(a,i2)'),trim(ix_str),ix
else
write(ix_str,'(i3)'),ix
endif
!
return
end subroutine get_3ch_string
!!
!!
subroutine convert_string_to_upper_or_lower_case(j,str,string)
!   =========================================================
!   Changes a string to upper case (j=1) or lower case (j=0) 
!   =========================================================
Implicit None
Integer, Intent(In)                  :: j
Character(*), Intent(In)             :: str
Character(LEN(str)),Intent(Out)      :: string
!
Integer :: ic, i

Character(26), Parameter :: cap = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
Character(26), Parameter :: low = 'abcdefghijklmnopqrstuvwxyz'

if (j==1) Then
!   Capitalize each letter if it is lowecase
    string = Trim(str)
    do i = 1, LEN_TRIM(str)
        ic = INDEX(low, str(i:i))
        if (ic > 0) string(i:i) = cap(ic:ic)
    end do
else if  (j==0) Then
!  from lower to upper case 
    string = Trim(str)
    do i = 1, LEN_TRIM(str)
        ic = INDEX(cap, str(i:i))
        if (ic > 0) string(i:i) = low(ic:ic)
    end do
else
write(*,*) 'problem:subr convert_string_to_upper_or_lower_case'
stop
endif
!
return
end subroutine convert_string_to_upper_or_lower_case
!!
subroutine EpochTime2ReadableFormat(epoch,t)
implicit none

Integer*4, intent(In)                     :: epoch
Integer*4, intent(Out)                    :: t(9)


CALL gmtime(epoch, t)
!write(*,737)  t(6)+1900, t(5)+1, t(4), t(3), t(2), t(1)

737 format(I4.4,I2.2,I2.2,".",I2.2,I2.2,I2.2)
return
end subroutine EpochTime2ReadableFormat
!!
subroutine GetOutFileDir(sacr_param,OutDirName)
Use radar_mods
Use write_messages
Use ifport
Implicit None
!
Type(sacr_parameters),Intent(In)                  :: sacr_param
Character(len=*),Intent(Out)                      :: OutDirName
! this subr gets the name of the output directory and creates it in the case it doesn't exist
!
logical                                        :: dir_e,dir_ex
character(len=356)                             :: err_message
character(len=356)                             :: outdir
character(len=356)                             :: dir_name
integer                                        :: status
logical(4)                                     :: lstatus
!
OutDirName=''
!
outdir=Trim(Adjustl(sacr_param%path_processed_data))//Trim(Adjustl(sacr_param%radar))//'/'

INQUIRE(DIRECTORY=outdir, EXIST=dir_e)
if (dir_e) then
continue
else
   ! if output directory doesn't exist, create it
   dir_name=outdir
   lstatus=makedirqq(Trim(dir_name))
   if (lstatus) then
    continue
   else
    err_message='Problem1: subr. GetOutFileDir'
    status=1; go to 999
   endif
endif

outdir=Trim(Adjustl(outdir))//Trim(Adjustl(sacr_param%mode))//'_data/'
INQUIRE(DIRECTORY=outdir, EXIST=dir_e)
if (dir_e) then
continue
else
   ! if output directory doesn't exist, create it
   dir_name=outdir
   lstatus=makedirqq(Trim(dir_name))
   if (lstatus) then
    continue
   else
    err_message='Problem2: subr. GetOutFileDir'
    status=1; go to 999
   endif
endif

outdir=Trim(Adjustl(outdir))//Trim(Adjustl(sacr_param%date))//'/'

INQUIRE(DIRECTORY=outdir, EXIST=dir_e)
if (dir_e) then
continue
else
   ! if output directory doesn't exist, create it
   dir_name=outdir
   lstatus=makedirqq(Trim(dir_name))
   if (lstatus) then
    continue
   else
    err_message='Problem3: subr. GetOutFileDir'
    status=1; go to 999
   endif
endif

999  If (status==1) Then
     Call write_error('Error in paths: - '//Trim(err_message))
     Call Exit(1)
     Endif
!
!
OutDirName=Trim(Adjustl(outdir))
!
return
end subroutine GetOutFileDir
!!
subroutine get_filename(long_name,filename)
  !
  Implicit none
  !
  !#
  !# Returns the filename given a full filename (path+filename)
  !#
  character(len=*),intent(in)  :: long_name
  character(len=*),intent(out) :: filename
  !
  !
  integer ibegstr
  integer iendstr
  integer i
  integer n
  logical endstrfound
  logical ispath
  !
  n=len(long_name)
  !
  !  
  ! find the actual length of the string
  !
  i=0
  endstrfound=.false.
  do while(.not.endstrfound)
     if(long_name(n-i:n).ne.' ') then
        iendstr=n-i+1
        endstrfound=.true.
     endif
     i=i+1
  enddo
  !
  iendstr=min(iendstr,n)
  !
  ! Is there a '/' ?
  !
  ispath=.false.
  do i=1,min(iendstr,n)
     if (long_name(i:i)=='/') then
        ispath=.true.
     endif
  enddo
  !
  if (ispath) then
     !  
     ! find the last '/'
     ! 
     i=iendstr
     do while(long_name(i:i).ne.'/')
        i=i-1
     enddo
     ibegstr=min(i+1,n)
     !
     !  path=long_name(1:ibegstr-1)
     !
     filename=long_name(ibegstr:iendstr)
     !
  else
     !
     filename=long_name(1:iendstr)
     !
  endif
  !
  return
end subroutine get_filename
!!
subroutine linpol ( x, y, imax, xp, yp, jmax )
Implicit None
!     
! Does a linear interpolation on the data given by the arrays
! x(1..imax) (x-coordinate) and
! y(1..imax) (y-coordinate)
!     
! Data are interpolated at the x-values given by the array
!xp(1..jmax) (x-coordinates of the interpolated values)
!     
! the interpolated y-values are returned in the array  yp(1..jmax)
! 
integer, Intent(In)       :: imax
real*8, Intent(In)        :: x(imax),y(imax)
integer, Intent(InOut)    :: jmax
real*8, Intent(InOut)     :: xp(jmax),yp(jmax)  
!
integer                   :: i,j,imin
!     
      imin=1
!
      do 1 j = 1, jmax
         if (xp(j) .lt. x(1)) then
            i=1
            yp(j)=(y(i+1)-y(i))/(x(i+1)-x(i))*(xp(j)-x(i)) + y(i)
         else if (xp(j) .gt. x(imax)) then
            i=imax-1
            yp(j)=(y(i+1)-y(i))/(x(i+1)-x(i))*(xp(j)-x(i)) + y(i)
         else if (xp(j) .eq. x(imax)) then
            yp(j)=y(imax)
         else
            do 2 i = imin, imax-1
               if (xp(j) .eq. x(i)) then
                  imin=i
                  yp(j)=y(i)
                  goto 1
               else if ((xp(j) .gt. x(i)).and.(xp(j) .le. x(i+1))) then
                  imin=i
                  yp(j)=(y(i+1)-y(i))/(x(i+1)-x(i))*(xp(j)-x(i)) + y(i)
                  goto 1
               endif
 2          continue
         endif
 1    continue
!     
return
end subroutine  linpol
!!
!!
subroutine linpol_sing(x,y,imax,xp,yp)
Implicit None    
! 
!Does a linear interpolation on the data given by the arrays
!   x(1..imax) (x-coordinate) and
!   y(1..imax) (y-coordinate)
!     
! Data are interpolated at the x-values given by the array
! xp(1) (x-coordinate of the interpolated values)
!     
! the interpolated y-value are returned in the array yp(1)
!    
Integer, Intent(In)         :: imax
Real*8, Intent(In)          :: x(imax),y(imax)
Real*8, Intent(InOut)       :: xp,yp
! 
integer                     ::  jmax, i, j,imin
!     
      imin=1
      jmax=1
!
      do 1 j = 1, jmax
         if (xp .lt. x(1)) then
            i=1
            yp=(y(i+1)-y(i))/(x(i+1)-x(i))*(xp-x(i)) + y(i)
         else if (xp .gt. x(imax)) then
            i=imax-1
            yp=(y(i+1)-y(i))/(x(i+1)-x(i))*(xp-x(i)) + y(i)
         else if (xp .eq. x(imax)) then
            yp=y(imax)
         else
            do 2 i = imin, imax-1
               if (xp .eq. x(i)) then
                  imin=i
                  yp=y(i)
                  goto 1
               else if ((xp .gt. x(i)).and.(xp .le. x(i+1))) then
                  imin=i
                  yp=(y(i+1)-y(i))/(x(i+1)-x(i))*(xp-x(i)) + y(i)
                  goto 1
               endif
 2          continue
         endif
 1    continue

return
end subroutine linpol_sing
!!
!!
subroutine CountLinesOfFile(InpFile,num_of_lines)
Implicit None
!
TYPE :: code
  CHARACTER (LEN=1140)  :: text
  TYPE (code), POINTER  :: next
END TYPE code

Character(len=*),Intent(in)                              :: InpFile
Integer, Intent(out)                                     :: num_of_lines
!
integer,parameter                                        :: id=1
character (len=1)                                        :: tab
type (code), pointer                                     :: head, current, tail, new_line, last_line
integer                                                  :: pos, last, iostatus, count, len_name, length, depth

tab = char(9)

open(unit=id,file=InpFile,status='old')
  ALLOCATE(head)
  tail => head
  READ(id, '(a)') head % text
  IF (head % text(1:1) == tab) head % text = head % text(2:)
  count = 1

  DO
    ALLOCATE(current)
    READ(id, '(a)', IOSTAT=iostatus) current % text
    IF (iostatus /= 0) EXIT
    count = count + 1
    DO
      pos = INDEX(current % text, tab)
      IF (pos > 0) THEN
        current % text = current % text(:pos-1) // '   ' // current % text(pos+1:)
      ELSE
        EXIT
      END IF
    END DO
    NULLIFY(current % next)
    tail % next => current
    tail => current
  END DO
close(id)

num_of_lines=count
!
return
end subroutine CountLinesOfFile
!!
subroutine get_env_vars(sacr_param)
Use write_messages
Use radar_mods
Implicit None
Type(sacr_parameters),Intent(Out)            :: sacr_param
!
integer                        :: status
character(len=80)              :: err_msg
character(len=6)               :: wstr
!
status=0
!
Call getenv('SACR_HOME', sacr_param%sacr_home)
if (Trim(sacr_param%sacr_home)=='') then ; status=1 ; err_msg='Error in environment variable sacr_home' ;  Goto 999 ; endif
!
Call getenv('radar', sacr_param%radar)
if (Trim(sacr_param%radar)=='') then ; status=1 ; err_msg='Error in environment variable sacr_home' ;  Goto 999 ; endif
!
Call getenv('location', sacr_param%location)
if (Trim(sacr_param%location)=='') then ; status=1 ; err_msg='Error in environment variable sacr_home' ;  Goto 999 ; endif
!
Call getenv('date', sacr_param%date)
if (Trim(sacr_param%date)=='') then ; status=1 ; err_msg='Error in environment variable sacr_home' ;  Goto 999 ; endif
!
Call getenv('mode', sacr_param%mode)
if (Trim(sacr_param%mode)=='') then ; status=1 ; err_msg='Error in environment variable sacr_home' ;  Goto 999 ; endif
!
Call getenv('path_noise_clear_sky', sacr_param%path_noise_clear_sky)
if (Trim(sacr_param%path_noise_clear_sky)=='') then ; status=1 ; err_msg='Error in environment variable sacr_home' ;  Goto 999 ; endif
!
wstr=''
Call getenv('csweep', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable csweep' ;  Goto 999 ; endif
read( wstr, '(i2)' ) sacr_param%csweep
!
if ((sacr_param%csweep/=0).and.(sacr_param%csweep/=1)) then ; status=1 ; err_msg='Error in environment variable csweep' ;  Goto 999 ; endif
!
Call getenv('path_processed_data',sacr_param%path_processed_data)
if (Trim(sacr_param%path_processed_data)=='') then ; status=1 ; err_msg='Error in environment variable sacr_home' ;  Goto 999 ; endif
!
!
wstr=''
Call getenv('ste_el_min', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable ste_el_min' ;  Goto 999 ; endif
read( wstr,'(f10.2)' ) sacr_param%ste_el_min
!
wstr=''
Call getenv('ste_el_max', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable ste_el_max' ;  Goto 999 ; endif
read( wstr,'(f10.2)' ) sacr_param%ste_el_max
!
wstr=''
Call getenv('ste_vel_thresh', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable ste_vel_thresh' ;  Goto 999 ; endif
read( wstr,'(f10.2)' ) sacr_param%ste_vel_thresh
!
wstr=''
Call getenv('ir_ldr_thresh', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable ir_ldr_thresh' ;  Goto 999 ; endif
read( wstr,'(f10.2)' ) sacr_param%ir_ldr_thresh
!
!
wstr=''
Call getenv('steID', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable steID' ;  Goto 999 ; endif
read( wstr, '(i2)' ) sacr_param%steID
!
wstr=''
Call getenv('irID', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable irID' ;  Goto 999 ; endif
read( wstr, '(i2)' ) sacr_param%irID
!
wstr=''
Call getenv('fmID', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable fmID' ;  Goto 999 ; endif
read( wstr, '(i2)' ) sacr_param%fmID
!
wstr=''
Call getenv('M1ID', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable M1ID' ;  Goto 999 ; endif
read( wstr, '(i2)' ) sacr_param%M1ID
!
wstr=''
Call getenv('M2ID', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable M2ID' ;  Goto 999 ; endif
read( wstr, '(i2)' ) sacr_param%M2ID
!
wstr=''
Call getenv('M3ID', wstr)
if (Trim(wstr)=='') then ; status=1 ; err_msg='Error in environment variable M3ID' ;  Goto 999 ; endif
read( wstr, '(i2)' ) sacr_param%M3ID
!


999 if (status /= 0) then ; Call write_error(err_msg) ; Call Exit(1) ; endif
!
return
end subroutine get_env_vars
!!
!!
!!
subroutine CorrectSweepIndices(modID,nt,azi,el,ns,faz,ij1,ij2)
Implicit None
!
Integer,Intent(In)                :: modID ! =0 (fixed azimuth,variable elevation; =1 fixed elevation, variable azimuth )
Integer,Intent(In)                :: nt
real*8,Intent(In)                 :: azi(nt)  ! azimuth
real*8,Intent(In)                 :: el(nt)  ! elevations
Integer,Intent(Out)               :: ns      ! number of scans
real*4,Intent(Out)                :: faz(nt) ! fixed angle
integer,Intent(Out)               :: ij1(nt),ij2(nt)   ! starting and ending sweep indices
!
integer                           :: it,is,ii,jj
integer,Dimension(:),Allocatable  :: incount  
!
!
real*8,Dimension(:),Allocatable   :: fixed_angle   ! fixed angle for scan
real*8,Dimension(:),Allocatable   :: fa_min,fa_max ! min,max fixed_angle in the central part of the scan

real*8,Dimension(:),Allocatable   :: const_angle  ! constant angle in the scan
real*8                            :: ca_minv,ca_maxv,del_ca ! [min,max, delta] (const_ang) 
!
real*8,Dimension(:),Allocatable   :: var_angle ! variable angle in the scan 
real*8,Dimension(:),Allocatable   :: va_min,va_max ! min,max variable angle in the scan
real*8                            :: va1,va2,va_width
real*8                            :: va_minv,va_maxv ! min,max variable angle in the range [va1,va2]

integer,Dimension(:),Allocatable  :: vmask 
integer                           :: it1,it2
real*8                            :: work,workd
!
!

! set missing values for fixed angle,sweep indices and number of scans
faz=-999.e0    ! 
ij1=-1; ij2=-1 ! 
ns=0  
!
!
Allocate(fixed_angle(nt),fa_min(nt),fa_max(nt))
fixed_angle=-999.d0; fa_min=-999.d0 ; fa_max=-999.d0 
!
Allocate(const_angle(nt))
Allocate(var_angle(nt),va_min(nt),va_max(nt))
va_min=-999.d0; va_max=-999.d0

!-------------------------------------
if (modID==0) then ! not ppimo
const_angle=azi
var_angle=el
else ! ppimo
const_angle=el
where(azi<0)    var_angle=360.d0-azi
where(azi>=0)   var_angle=360.d0+azi
endif
!-------------------------------------

! estimate the number of scans, fixed angle and indices using elevations and azimuth
!
work=0.75d0*(0.5d0*(MaxVal(var_angle)-MinVal(var_angle)))
workd=0.5d0*(MaxVal(var_angle)+MinVal(var_angle))
va1=workd-work  ! lowest var_angle to look for
va2=workd+work  ! highest var_angle to look for
va_width=0.9d0*(va2-va1) ! var_angle span we are going to search in data
del_ca=0.55d0 ! we are looking at data at approx fixed angle range fix_angle-del_az, fix_angle+del_az
!
Allocate(vmask(nt))
Allocate(incount(nt))
!
vmask=0;incount=-999
!
! define mask vmask. vmask=1 if variable angle values are between va1 and va2; otherwise =0 
do it=1,nt
   if ((var_angle(it)>=va1).and.(var_angle(it)<=va2)) then
      vmask(it)=1
    endif
enddo
!
! if ppimo go back to initial values for azimuth
if (modID==1) var_angle=azi
!! 
!! there are now  two possibilities:
!! 1. the first value of not-fixed angle is not between va1,va2, 
!! 2. the first value of not-fixed angle is already between va1,va2 
!! 

is=0  ! this is the scan number index

! 
if (vmask(1)==0) then   ! if the first value of not-fixed angle is not between va1,va2
  do it=1,nt-1
      if ((vmask(it)==0).and.(vmask(it+1)==1)) then  ! we look for situation vmask=0-1 in 0-1-1-1....-0 case
          if (modID==0) const_angle=azi
          if (modID==1) then
                where(azi<0)    var_angle=360.d0-azi
                where(azi>=0)   var_angle=360.d0+azi
           endif


               it1=it+1       !  el1 index
               it2=nt
                 M1: do ii=it1,nt-1
                        if ((vmask(ii)==1).and.(vmask(ii+1)==0)) then
                           it2=ii  ! el2 index
                           Exit M1
                        end if
                 enddo M1


           va_minv=MinVal(var_angle(it1:it2))  ! minimum elevation
           va_maxv=MaxVal(var_angle(it1:it2))  ! maximum elevation
           ca_minv=MinVal(const_angle(it1:it2))  ! minimum azimuth 
           ca_maxv=MaxVal(const_angle(it1:it2))  ! maximum azimuth


           if ( (ca_minv<=del_ca*0.5d0) .and. (ca_maxv>=360.d0-del_ca*0.5d0)) then
              do ii=it1,it2
                 if (const_angle(ii)>=0.d0) const_angle(ii)=360.d0+const_angle(ii)
              enddo    
                  ca_minv=MinVal(const_angle(it1:it2))  ! minimum azimuth
                  ca_maxv=MaxVal(const_angle(it1:it2))  ! maximum azimuth
            endif
       
           


!  we are looking at actual scan IF
!  1. difference between max and min elevation found is larger than va_width and
!  2. difference between max and min azimuth is smaller than del_ca degrees 
           if (((va_maxv-va_minv)>=va_width).and.(ca_maxv-ca_minv<=del_ca)) then

           is=is+1 ! scan number index

           incount(is)=(it2-it1+1)
           fixed_angle(is)=Sum(const_angle(it1:it2))/dble(it2-it1+1)  ! fixed azimuth
           fa_min(is)=fixed_angle(is)-del_ca*0.5d0
           fa_max(is)=fixed_angle(is)+del_ca*0.5d0
           va_min(is)=va_minv
           va_max(is)=va_maxv


                     if (modID==1) then

                          S1: do ii=Max(it1-1,1),1,-1
                                 if(azi(ii+1)-azi(ii) <=0) Exit S1
                              enddo S1
                              va_minv=var_angle(ii+1)

                              jj=it2
                          L1:do ii=Min(it2+1,nt),nt
                              if (azi(ii)-azi(ii-1)<=0) then
                                 jj=ii
                                 exit L1
                               endif
                              enddo L1
                           
                           H1: do ii=jj,nt
                                 if ((azi(ii)>=0).and.(azi(ii)<=va_minv-360.d0)) then
                                 var_angle(ii)=360.d0+var_angle(ii)
                                 else
                                 exit H1
                                 endif
                           enddo H1
                               va_maxv=var_angle(ii-1)
                        endif

                 !----------------------------------------------------------------------------------
                 ! find the sweep indices - actual indices are ij1(<= it1) and ij2(>=it2)
                 call GetSweepIndices(nt,it1,it2,fixed_angle(is),fa_min(is),fa_max(is),va_min(is),va_max(is),const_angle,var_angle,ij1(is),ij2(is))
                 !----------------------------------------------------------------------------------

           endif
       endif
  enddo
endif

if (vmask(1)==1) then ! ! if the first value of variable scan angle  is between va1,va2
  do it=1,nt-1
      if ((vmask(it)==1).and.(vmask(it+1)==0)) then  ! we look for situation vmask=[1-0] in 1-1-1-1...[1-0] case
          if (modID==0) const_angle=azi
          if (modID==1) then
                where(azi<0)    var_angle=360.d0-azi
                where(azi>=0)   var_angle=360.d0+azi
           endif


               it2=it   ! el2 index
               it1=1
                 M2: do ii=it2,2,-1
                        if ((vmask(ii)==1).and.(vmask(ii-1)==0)) then
                           it1=ii    ! el1 index
                           Exit M2
                        end if
                 enddo M2

            
           va_minv=MinVal(var_angle(it1:it2))  !  minimum elevation  
           va_maxv=MaxVal(var_angle(it1:it2))  !  maximum elevation
           ca_minv=MinVal(const_angle(it1:it2))  !  minimum azimuth
           ca_maxv=MaxVal(const_angle(it1:it2))  !  maximum azimuth 

           
           if ( (ca_minv<=del_ca*0.5d0) .and. (ca_maxv>=360.d0-del_ca*0.5d0)) then
              do ii=it1,it2
                 if (const_angle(ii)>=0.d0) const_angle(ii)=360.d0+const_angle(ii)
              enddo
                  ca_minv=MinVal(const_angle(it1:it2))  ! minimum azimuth
                  ca_maxv=MaxVal(const_angle(it1:it2))  ! maximum azimuth
            endif

!----------------------------------------------------------------------------------
!  we are looking at actual scan IF
!  1. difference between max and min elevation found is larger than va_width and
!  2. difference between max and min azimuth is smaller than del_ca degrees  
           if (((va_maxv-va_minv)>=va_width).and.(ca_maxv-ca_minv<=del_ca)) then
  
           is=is+1  ! scan number index
           incount(is)=(it2-it1+1)  ! number of data in scan
           fixed_angle(is)=Sum(const_angle(it1:it2))/dble(it2-it1+1)  ! fixed azimuth value
           fa_min(is)=fixed_angle(is)-0.5d0*del_ca
           fa_max(is)=fixed_angle(is)+0.5d0*del_ca
           va_min(is)=va_minv
           va_max(is)=va_maxv

                     if (modID==1) then

                          S2: do ii=Max(it1-1,1),1,-1
                                 if(azi(ii+1)-azi(ii) <=0) Exit S2
                              enddo S2
                              va_minv=var_angle(ii+1)

                              jj=it2
                          L2:do ii=Min(it2+1,nt),nt
                              if (azi(ii)-azi(ii-1)<=0) then
                                 jj=ii
                                 exit L2
                               endif
                              enddo L2

                           H2: do ii=jj,nt
                                 if ((azi(ii)>=0).and.(azi(ii)<=va_minv-360.d0)) then
                                 var_angle(ii)=360.d0+var_angle(ii)
                                 else
                                 exit H2
                                 endif
                           enddo H2
                               va_maxv=var_angle(ii-1)
                        endif

                 !----------------------------------------------------------------------------------
                 ! find the sweep indices - actual indices are ij1(<= it1) and ij2(>=it2)
                 call GetSweepIndices(nt,it1,it2,fixed_angle(is),fa_min(is),fa_max(is),va_min(is),va_max(is),const_angle,var_angle,ij1(is),ij2(is))
                 !----------------------------------------------------------------------------------

           endif ! if (((el_max-el_min)>=va_width).and.(az_max-az_min<=3))

!----------------------------------------------------------------------------------

     endif ! if ((vmask(it)==1).and.(vmask(it+1)==0))
  enddo    
endif


ns=is  ! total number of scan is equal to the last counting index is
!! the first ns values will have actual values for all output vars, and missing values in range ns+1:nt
faz(1:ns)=real(fixed_angle(1:ns)) ! fixed azimuth
!
!
Deallocate(vmask,fixed_angle,fa_min,fa_max,const_angle,var_angle,va_min,va_max,incount)
!
return
end subroutine CorrectSweepIndices
!
!!
subroutine GetSweepIndices(nt,it1,it2,fix_angle,azmin,azmax,elmin,elmax,az,el,ij1,ij2)
Implicit None
!
Integer, Intent(In)            :: nt
Integer, Intent(In)            :: it1,it2
Real*8, Intent(In)             :: fix_angle
Real*8, Intent(In)             :: azmin,azmax
Real*8, Intent(In)             :: elmin,elmax
Real*8, Intent(In)             :: az(nt),el(nt)
Integer, Intent(Out)           :: ij1,ij2
!
integer                        :: it
integer                        :: elID
integer                        :: j,j1,j2

!
ij1=-1  ; ij2=-1
!
! check if elevation increase or decrease
elID=1 ! assume elevations increase
if (el(it1)>el(it2)) elID=-1 ! el decrease
!
!-----------------------------------------------------
! case when elevations increase in time
!-----------------------------------------------------

if ( elID==1 ) then
!
          ! look elevations bellow elmin
          ij1=it1
          if (it1>1) Then  
            j1=2 ; j2=it1
             A1:do j=j2,j1,-1
                   !if (el(j)>el(j-1))  then
                   if ((el(j)>el(j-1)) .and. (az(j-1)>=azmin).and.(az(j-1)<=azmax)) then   
                     ij1=j-1
                     else
                     exit A1
                   endif
                enddo A1
           endif
!
          ! look elevations above elmax
          ij2=it2
          if (it2<nt) Then
            j1=it2 ; j2=nt-1
             A2:do j=j1,j2
                   !if (el(j+1)>el(j)) then
                   if ((el(j+1)>el(j)) .and.(az(j+1)>=azmin).and.(az(j+1)<=azmax)) then
                     ij2=j+1
                     else
                     exit A2
                   endif
                enddo A2
           endif
!
endif
!-------------------------------------------------------
! case when elevations decrease in time
!-----------------------------------------------------

if ( elID==-1 ) then
!
          ! look elevations above elmax
          ij1=it1
          if (it1>1) Then
            j1=2 ; j2=it1
             B1:do j=j2,j1,-1
                   !if (el(j)<el(j-1)) then
                   if ((el(j)<el(j-1)) .and. (az(j-1)>=azmin).and.(az(j-1)<=azmax)) then
                     ij1=j-1
                     else
                     exit B1
                   endif
                enddo B1
           endif
!
          ! look elevations below elmin
          ij2=it2
          if (it2<nt) Then
            j1=it2 ; j2=nt-1
             B2:do j=j1,j2
                   !if (el(j+1)<el(j)) then
                   if ((el(j+1)<el(j)) .and. (az(j+1)>=azmin).and.(az(j+1)<=azmax)) then
                     ij2=j+1
                     else
                     exit B2
                   endif
                enddo B2
           endif
!
endif
!-------------------------------------------------------

go to 234
!
j1=Max(1,ij1-10)
j2=Min(nt,ij2+10)
!
write(*,*) ' '
do j=j1,j2
if (j==ij1) write(*,*) '-------------------------------------------'
write(*,*) j,real(el(j)),real(azmin),real(az(j)),real(azmax)
if (j==ij2) write(*,*) '-------------------------------------------'
enddo
write(*,*) ' '
pause
!
234 continue
!
return
end subroutine GetSweepIndices 





