#! /bin/sh
#set -eax
set -e +ax

# configuration for spectrum
#export LD_LIBRARY_PATH=/aos/shared/lib/centos6/netcdf/4/intel/lib:$LD_LIBRARY_PATH
#export LD_LIBRARY_PATH=/aos/shared/lib/centos6/hdf5/default/intel/lib:$LD_LIBRARY_PATH
#____________________________________________________________________________________________________________________________
#____________________________________________________________________________________________________________________________
#
#  DEFINE PARAMETERS
#____________________________________________________________________________________________________________________________
#____________________________________________________________________________________________________________________________
# radar  must be either KaSACR or WbSACR (or XaSACR for TMP)
export radar="KaSACR"
#
#
# location SGP,NSA,PVC,TMP 
export location="SGP"
#
# date
#export date="20120211" # ppi
export date="20110526"   
#
#
# mode must be hsrhi,blrhi, vptmo, ppimo, cwrhi,awrhi
export mode="hsrhi"
#
# apply the correction for the sweep indices =1 yes,=0 no
export csweep="1"
# elevation range in degrees not to be checked for the second trip echo 
export ste_el_min="15.0"
export ste_el_max="165.0"
# treshold value of velocity standard deviation in m/s for the second trip echo identification,
#  if std > std treshold the cloud pixel is identified as the sec.trip echo
export ste_vel_thresh="1.5" 
# LDR treshold in dB for the insect removal below min(first cloud base, 5 degrees izotherm), insects if LDR>LDR threshoold
# LDR valid range =-5 to  -15
export ir_ldr_thresh="-10.0" 
#
#_______________________________________
# Define data paths accordingly
export SACR_HOME="/storage/aleksandra/starbucks/sacr/"
export path_noise_clear_sky="/storage/aleksandra/starbucks/sacr/sacr_aux/"
path_common_data="/storage2/common_data/"
path_raw_data="${path_common_data}${location}/raw_data/${radar}/${mode}_data/"
###
# the following paths may be defined "NULL" if you want to exclude them
path_mergedsonde_data="${path_common_data}${location}/raw_data/mergedsonde_data/"
path_sounding_data="${path_common_data}${location}/raw_data/sounding_data/"
path_ceilo_data="${path_common_data}${location}/raw_data/ceilo_data/"
#
#
#path_mergedsonde_data="NULL"
#path_sounding_data="NULL"
#path_ceilo_data="NULL"
#
#
export path_processed_data="${SACR_HOME}TEST_OUT/${location}/processed_data/"
#____________________________________________________________________________________________________________________________
#____________________________________________________________________________________________________________________________
#    END DEFINE PARAMETERS
#____________________________________________________________________________________________________________________________
#____________________________________________________________________________________________________________________________
#
#
# options for testing - don't change values 
# whether or not to apply the insect removal, =1 yes,=0 no
export irID="1"
# whether or not to apply the second trip echo removal, =1 yes,=0 no
export steID="1"
# whether or not to apply the mask1 (removing noise pixels using H&S74 and/or climat.noise values), =1 yes,=0 no
export M1ID="1"
# whether or not to apply the ldr mask, =1 yes,=0 no
export M2ID="1"
# whether or not to apply the masking using the ibox filter, =1 yes,=0 no
export M3ID="1"
# whether or not to apply the final masking, =1 yes,=0 no
export fmID="1"
#
#____________________________________________________________________________________________________________________________
# create the output directory $path_processed_data
mkdir -p $path_processed_data
#
#--------------------------------------------------------------------
# check if specified directories exist
status=1
for dir in $SACR_HOME $path_noise_clear_sky $path_common_data $path_raw_data $path_processed_data ; do
if [ ! -d "$dir" ]; then
status=0
echo "Directory $dir does not exist." 
fi
done
#
if [ $status -eq 0 ]; then
    echo "Error.Correct the paths."
    exit 1
fi
#
#--------------------------------------------------------------------
# create input file path_raw_data.list
ls ${path_raw_data}*${date}*.nc > temp.list
if [ `ls -l temp.list | awk '{print $5}'` -eq 0 ]; then
    echo "Error.Directory $path_raw_data does not contain files ${date}*.nc."
    exit 1
fi

awk < temp.list '{ print "\"" $1"\"" }' > path_raw_data_${location}_${radar}_${mode}_${date}.list

#--------------------------------------------------------------------
# SOUNDING DATA
#--------------------------------------------------------------------
# if $path_sounding_data == "NULL"  -> write "NULL" into *list file 
if [ "$path_sounding_data" == "NULL" ]; then
echo "STATUS: path_sounding_data specified as NULL." 
echo "\""NULL"\"" > path_sounding_data_${location}_${radar}_${mode}_${date}.list
fi
#
# if $path_sounding_data /= "NULL" 
if [ "$path_sounding_data" != "NULL" ]; then
     # check if directory exist
     dir="$path_sounding_data"
     if [ ! -d "$dir" ]; then  # if does not exist Exit
         echo "Directory $dir does not exist." 
         echo "Error.Correct the path for sounding data."
         exit 1
     else  # if directory exist check if it contains required input file
         entry=${path_sounding_data}*${date}*.cdf
            for i in $entry; { [ -e "$i" ] && break || i=”; }
            #[ "$i" ] 
         if [ ! -e $i ]; then  
              # required input file does not exist -> write "NULL" into *list file
            echo "\""NULL"\"" > path_sounding_data_${location}_${radar}_${mode}_${date}.list
         else #write files into *list file
            ls ${path_sounding_data}*${date}*.cdf > temp.list
            awk < temp.list '{ print "\"" $1"\"" }' > path_sounding_data_${location}_${radar}_${mode}_${date}.list
         fi
      fi
fi

#--------------------------------------------------------------------
#--------------------------------------------------------------------
# MERGED SOUNDING DATA
#--------------------------------------------------------------------
# if $path_mergedsonde_data == "NULL"  -> write "NULL" into *list file 
if [ "$path_mergedsonde_data" == "NULL" ]; then
echo "STATUS: path_mergedsonde_data specified as NULL." 
echo "\""NULL"\"" > path_mergedsonde_data_${location}_${radar}_${mode}_${date}.list
fi
#
# if $path_mergedsonde_data /= "NULL" 
if [ "$path_mergedsonde_data" != "NULL" ]; then
     # check if directory exist
     dir="$path_mergedsonde_data"
     if [ ! -d "$dir" ]; then  # if does not exist Exit
         echo "Directory $dir does not exist." 
         echo "Error.Correct the path for mergedsonde data."
         exit 1
     else  # if directory exist check if it contains required input file
         entry=${path_mergedsonde_data}*${date}*.cdf       
         if [ -f $entry ];then    # yes, write files into *list file
            ls ${path_mergedsonde_data}*${date}*.cdf > temp.list
            awk < temp.list '{ print "\"" $1"\"" }' > path_mergedsonde_data_${location}_${radar}_${mode}_${date}.list
         else # required input file does not exist -> write "NULL" into *list file
            echo "\""NULL"\"" > path_mergedsonde_data_${location}_${radar}_${mode}_${date}.list
         fi
      fi
fi
#--------------------------------------------------------------------
#--------------------------------------------------------------------
# CEILOMETER DATA
#--------------------------------------------------------------------    
# if $path_ceilo_data == "NULL"  -> write "NULL" into *list file 
if [ "$path_ceilo_data" == "NULL" ]; then
echo "STATUS: path_ceilo_data specified as NULL." 
echo "\""NULL"\"" > path_ceilometer_data_${location}_${radar}_${mode}_${date}.list
fi
#
# if $path_ceilo_data /= "NULL" 
if [ "$path_ceilo_data" != "NULL" ]; then
     # check if directory exist
     dir="$path_ceilo_data"
     if [ ! -d "$dir" ]; then  # if does not exist Exit
         echo "Directory $dir does not exist." 
         echo "Error.Correct the path for ceilometer data."
         exit 1
     else  # if directory exist check if it contains required input file
         entry=${path_ceilo_data}*${date}*.cdf       
         if [ -f $entry ];then    # yes, write files into *list file
            ls ${path_ceilo_data}*${date}*.cdf > temp.list
            awk < temp.list '{ print "\"" $1"\"" }' > path_ceilometer_data_${location}_${radar}_${mode}_${date}.list
         else # required input file does not exist -> write "NULL" into *list file
            echo "\""NULL"\"" > path_ceilometer_data_${location}_${radar}_${mode}_${date}.list
         fi
      fi
fi
#--------------------------------------------------------------------
#--------------------------------------------------------------------
rm -f temp.list
# run sacr fortran code
./sacr
#--------------------------------------------------------------------
#--------------------------------------------------------------------
