!! ----------------------------------------------------------------------------
!! *PROGRAM* sort_mod
!! @version sacr 1.0
!!
!! *SRC_FILE*
!! sacr/sort_mod.f90
!!
!!
!! *LAST CHANGES*
!!
!!
!! *DESCRIPTION* 
!!
!! This module contains subroutines needed for data  sorting 
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
!!

MODULE  Sorting
   IMPLICIT  NONE
   PUBLIC   :: FindMinimum,FindMaximum,Swap,Sort

CONTAINS

! --------------------------------------------------------------------
! INTEGER FUNCTION  FindMinimum():
!    This function returns the location of the minimum in the section
! between Start and End.
! --------------------------------------------------------------------

   INTEGER FUNCTION  FindMinimum(x, Start, End)
      IMPLICIT  NONE
      REAL*8, DIMENSION(1:), INTENT(IN) :: x
      INTEGER, INTENT(IN)                :: Start, End
      Real*8                            :: Minimum
      INTEGER                            :: Location
      INTEGER                            :: i

      Minimum  = x(Start)          ! assume the first is the min
      Location = Start             ! record its position
      DO i = Start+1, End          ! start with next elements
         IF (x(i) < Minimum) THEN  !   if x(i) less than the min?
            Minimum  = x(i)        !      Yes, a new minimum found
            Location = i                !      record its position
         END IF
      END DO
      FindMinimum = Location            ! return the position
   END FUNCTION  FindMinimum
! --------------------------------------------------------------------
! INTEGER FUNCTION  FindMaximum():
!    This function returns the location of the maximum in the section
! between Start and End.
! --------------------------------------------------------------------

   INTEGER FUNCTION  FindMaximum(x, Start, End)
      IMPLICIT  NONE
      REAL*8, DIMENSION(1:), INTENT(IN) :: x
      INTEGER, INTENT(IN)                :: Start, End
      Real*8                            :: Maximum
      INTEGER                            :: Location
      INTEGER                            :: i

      Maximum  = x(Start)          ! assume the first is the max
      Location = Start             ! record its position
      DO i = Start+1, End          ! start with next elements
         IF (x(i) > Maximum) THEN  !   if x(i) greater than the max?
            Maximum  = x(i)        !      Yes, a new maximum found
            Location = i                !      record its position
         END IF
      END DO
      FindMaximum = Location            ! return the position
   END FUNCTION  FindMaximum
!
! --------------------------------------------------------------------
! SUBROUTINE  Swap():
!    This subroutine swaps the values of its two formal arguments.
! --------------------------------------------------------------------

   SUBROUTINE  Swap(a, b)
      IMPLICIT  NONE
      REAL*8, INTENT(INOUT) :: a, b
      REAL*8                :: Temp

      Temp = a
      a    = b
      b    = Temp
   END SUBROUTINE  Swap

! --------------------------------------------------------------------
! SUBROUTINE  Sort():
!    This subroutine receives an array x() and sorts it into ascending
! order.
! --------------------------------------------------------------------

   SUBROUTINE  Sort(x, Size)
      IMPLICIT  NONE
      REAL*8, DIMENSION(1:), INTENT(INOUT) :: x
      INTEGER, INTENT(IN)                   :: Size
      INTEGER                               :: i
      INTEGER                               :: Location

      DO i = 1, Size-1             ! except for the last
         Location = FindMinimum(x, i, Size)  ! find min from this to last
         CALL  Swap(x(i), x(Location))  ! swap this and the minimum
      END DO
   END SUBROUTINE  Sort

END MODULE  Sorting
